# Copyright (c) 2014 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
from boto.compat import json
from boto.exception import JSONResponseError
from boto.connection import AWSAuthConnection
from boto.regioninfo import RegionInfo
from boto.cognito.sync import exceptions


class CognitoSyncConnection(AWSAuthConnection):
    """
    Amazon Cognito Sync
    Amazon Cognito Sync provides an AWS service and client library
    that enable cross-device syncing of application-related user data.
    High-level client libraries are available for both iOS and
    Android. You can use these libraries to persist data locally so
    that it's available even if the device is offline. Developer
    credentials don't need to be stored on the mobile device to access
    the service. You can use Amazon Cognito to obtain a normalized
    user ID and credentials. User data is persisted in a dataset that
    can store up to 1 MB of key-value pairs, and you can have up to 20
    datasets per user identity.

    With Amazon Cognito Sync, the data stored for each identity is
    accessible only to credentials assigned to that identity. In order
    to use the Cognito Sync service, you need to make API calls using
    credentials retrieved with `Amazon Cognito Identity service`_.
    """
    APIVersion = "2014-06-30"
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "cognito-sync.us-east-1.amazonaws.com"
    ResponseError = JSONResponseError

    _faults = {
        "LimitExceededException": exceptions.LimitExceededException,
        "ResourceConflictException": exceptions.ResourceConflictException,
        "InvalidConfigurationException": exceptions.InvalidConfigurationException,
        "TooManyRequestsException": exceptions.TooManyRequestsException,
        "InvalidParameterException": exceptions.InvalidParameterException,
        "ResourceNotFoundException": exceptions.ResourceNotFoundException,
        "InternalErrorException": exceptions.InternalErrorException,
        "NotAuthorizedException": exceptions.NotAuthorizedException,
    }


    def __init__(self, **kwargs):
        region = kwargs.get('region')
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)
        else:
            del kwargs['region']
        kwargs['host'] = region.endpoint
        super(CognitoSyncConnection, self).__init__(**kwargs)
        self.region = region

    def _required_auth_capability(self):
        return ['hmac-v4']

    def delete_dataset(self, identity_pool_id, identity_id, dataset_name):
        """
        Deletes the specific dataset. The dataset will be deleted
        permanently, and the action can't be undone. Datasets that
        this dataset was merged with will no longer report the merge.
        Any consequent operation on this dataset will result in a
        ResourceNotFoundException.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type identity_id: string
        :param identity_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type dataset_name: string
        :param dataset_name: A string of up to 128 characters. Allowed
            characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.'
            (dot).

        """

        uri = '/identitypools/{0}/identities/{1}/datasets/{2}'.format(
            identity_pool_id, identity_id, dataset_name)
        return self.make_request('DELETE', uri, expected_status=200)

    def describe_dataset(self, identity_pool_id, identity_id, dataset_name):
        """
        Gets metadata about a dataset by identity and dataset name.
        The credentials used to make this API call need to have access
        to the identity data. With Amazon Cognito Sync, each identity
        has access only to its own data. You should use Amazon Cognito
        Identity service to retrieve the credentials necessary to make
        this API call.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type identity_id: string
        :param identity_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type dataset_name: string
        :param dataset_name: A string of up to 128 characters. Allowed
            characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.'
            (dot).

        """

        uri = '/identitypools/{0}/identities/{1}/datasets/{2}'.format(
            identity_pool_id, identity_id, dataset_name)
        return self.make_request('GET', uri, expected_status=200)

    def describe_identity_pool_usage(self, identity_pool_id):
        """
        Gets usage details (for example, data storage) about a
        particular identity pool.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        """

        uri = '/identitypools/{0}'.format(identity_pool_id)
        return self.make_request('GET', uri, expected_status=200)

    def describe_identity_usage(self, identity_pool_id, identity_id):
        """
        Gets usage information for an identity, including number of
        datasets and data usage.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type identity_id: string
        :param identity_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        """

        uri = '/identitypools/{0}/identities/{1}'.format(
            identity_pool_id, identity_id)
        return self.make_request('GET', uri, expected_status=200)

    def get_identity_pool_configuration(self, identity_pool_id):
        """
        Gets the configuration settings of an identity pool.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. This is the ID of the pool for which to return a
            configuration.

        """

        uri = '/identitypools/{0}/configuration'.format(identity_pool_id)
        return self.make_request('GET', uri, expected_status=200)

    def list_datasets(self, identity_pool_id, identity_id, next_token=None,
                      max_results=None):
        """
        Lists datasets for an identity. The credentials used to make
        this API call need to have access to the identity data. With
        Amazon Cognito Sync, each identity has access only to its own
        data. You should use Amazon Cognito Identity service to
        retrieve the credentials necessary to make this API call.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type identity_id: string
        :param identity_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type next_token: string
        :param next_token: A pagination token for obtaining the next page of
            results.

        :type max_results: integer
        :param max_results: The maximum number of results to be returned.

        """

        uri = '/identitypools/{0}/identities/{1}/datasets'.format(
            identity_pool_id, identity_id)
        params = {}
        headers = {}
        query_params = {}
        if next_token is not None:
            query_params['nextToken'] = next_token
        if max_results is not None:
            query_params['maxResults'] = max_results
        return self.make_request('GET', uri, expected_status=200,
                                 data=json.dumps(params), headers=headers,
                                 params=query_params)

    def list_identity_pool_usage(self, next_token=None, max_results=None):
        """
        Gets a list of identity pools registered with Cognito.

        :type next_token: string
        :param next_token: A pagination token for obtaining the next page of
            results.

        :type max_results: integer
        :param max_results: The maximum number of results to be returned.

        """

        uri = '/identitypools'
        params = {}
        headers = {}
        query_params = {}
        if next_token is not None:
            query_params['nextToken'] = next_token
        if max_results is not None:
            query_params['maxResults'] = max_results
        return self.make_request('GET', uri, expected_status=200,
                                 data=json.dumps(params), headers=headers,
                                 params=query_params)

    def list_records(self, identity_pool_id, identity_id, dataset_name,
                     last_sync_count=None, next_token=None, max_results=None,
                     sync_session_token=None):
        """
        Gets paginated records, optionally changed after a particular
        sync count for a dataset and identity. The credentials used to
        make this API call need to have access to the identity data.
        With Amazon Cognito Sync, each identity has access only to its
        own data. You should use Amazon Cognito Identity service to
        retrieve the credentials necessary to make this API call.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type identity_id: string
        :param identity_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type dataset_name: string
        :param dataset_name: A string of up to 128 characters. Allowed
            characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.'
            (dot).

        :type last_sync_count: long
        :param last_sync_count: The last server sync count for this record.

        :type next_token: string
        :param next_token: A pagination token for obtaining the next page of
            results.

        :type max_results: integer
        :param max_results: The maximum number of results to be returned.

        :type sync_session_token: string
        :param sync_session_token: A token containing a session ID, identity
            ID, and expiration.

        """

        uri = '/identitypools/{0}/identities/{1}/datasets/{2}/records'.format(
            identity_pool_id, identity_id, dataset_name)
        params = {}
        headers = {}
        query_params = {}
        if last_sync_count is not None:
            query_params['lastSyncCount'] = last_sync_count
        if next_token is not None:
            query_params['nextToken'] = next_token
        if max_results is not None:
            query_params['maxResults'] = max_results
        if sync_session_token is not None:
            query_params['syncSessionToken'] = sync_session_token
        return self.make_request('GET', uri, expected_status=200,
                                 data=json.dumps(params), headers=headers,
                                 params=query_params)

    def register_device(self, identity_pool_id, identity_id, platform, token):
        """
        Registers a device to receive push sync notifications.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. Here, the ID of the pool that the identity belongs to.

        :type identity_id: string
        :param identity_id: The unique ID for this identity.

        :type platform: string
        :param platform: The SNS platform type (e.g. GCM, SDM, APNS,
            APNS_SANDBOX).

        :type token: string
        :param token: The push token.

        """

        uri = '/identitypools/{0}/identity/{1}/device'.format(
            identity_pool_id, identity_id)
        params = {'Platform': platform, 'Token': token, }
        headers = {}
        query_params = {}
        return self.make_request('POST', uri, expected_status=200,
                                 data=json.dumps(params), headers=headers,
                                 params=query_params)

    def set_identity_pool_configuration(self, identity_pool_id,
                                        push_sync=None):
        """
        Sets the necessary configuration for push sync.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. This is the ID of the pool to modify.

        :type push_sync: dict
        :param push_sync: Configuration options to be applied to the identity
            pool.

        """

        uri = '/identitypools/{0}/configuration'.format(identity_pool_id)
        params = {}
        headers = {}
        query_params = {}
        if push_sync is not None:
            params['PushSync'] = push_sync
        return self.make_request('POST', uri, expected_status=200,
                                 data=json.dumps(params), headers=headers,
                                 params=query_params)

    def subscribe_to_dataset(self, identity_pool_id, identity_id,
                             dataset_name, device_id):
        """
        Subscribes to receive notifications when a dataset is modified
        by another device.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. The ID of the pool to which the identity belongs.

        :type identity_id: string
        :param identity_id: Unique ID for this identity.

        :type dataset_name: string
        :param dataset_name: The name of the dataset to subcribe to.

        :type device_id: string
        :param device_id: The unique ID generated for this device by Cognito.

        """

        uri = '/identitypools/{0}/identities/{1}/datasets/{2}/subscriptions/{3}'.format(
            identity_pool_id, identity_id, dataset_name, device_id)
        return self.make_request('POST', uri, expected_status=200)

    def unsubscribe_from_dataset(self, identity_pool_id, identity_id,
                                 dataset_name, device_id):
        """
        Unsubscribe from receiving notifications when a dataset is
        modified by another device.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. The ID of the pool to which this identity belongs.

        :type identity_id: string
        :param identity_id: Unique ID for this identity.

        :type dataset_name: string
        :param dataset_name: The name of the dataset from which to unsubcribe.

        :type device_id: string
        :param device_id: The unique ID generated for this device by Cognito.

        """

        uri = '/identitypools/{0}/identities/{1}/datasets/{2}/subscriptions/{3}'.format(
            identity_pool_id, identity_id, dataset_name, device_id)
        return self.make_request('DELETE', uri, expected_status=200)

    def update_records(self, identity_pool_id, identity_id, dataset_name,
                       sync_session_token, device_id=None,
                       record_patches=None, client_context=None):
        """
        Posts updates to records and add and delete records for a
        dataset and user. The credentials used to make this API call
        need to have access to the identity data. With Amazon Cognito
        Sync, each identity has access only to its own data. You
        should use Amazon Cognito Identity service to retrieve the
        credentials necessary to make this API call.

        :type identity_pool_id: string
        :param identity_pool_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type identity_id: string
        :param identity_id: A name-spaced GUID (for example, us-
            east-1:23EC4050-6AEA-7089-A2DD-08002EXAMPLE) created by Amazon
            Cognito. GUID generation is unique within a region.

        :type dataset_name: string
        :param dataset_name: A string of up to 128 characters. Allowed
            characters are a-z, A-Z, 0-9, '_' (underscore), '-' (dash), and '.'
            (dot).

        :type device_id: string
        :param device_id: The unique ID generated for this device by Cognito.

        :type record_patches: list
        :param record_patches: A list of patch operations.

        :type sync_session_token: string
        :param sync_session_token: The SyncSessionToken returned by a previous
            call to ListRecords for this dataset and identity.

        :type client_context: string
        :param client_context: Intended to supply a device ID that will
            populate the `lastModifiedBy` field referenced in other methods.
            The `ClientContext` field is not yet implemented.

        """

        uri = '/identitypools/{0}/identities/{1}/datasets/{2}'.format(
            identity_pool_id, identity_id, dataset_name)
        params = {'SyncSessionToken': sync_session_token, }
        headers = {}
        query_params = {}
        if device_id is not None:
            params['DeviceId'] = device_id
        if record_patches is not None:
            params['RecordPatches'] = record_patches
        if client_context is not None:
            headers['x-amz-Client-Context'] = client_context
        if client_context is not None:
            headers['x-amz-Client-Context'] = client_context
        return self.make_request('POST', uri, expected_status=200,
                                 data=json.dumps(params), headers=headers,
                                 params=query_params)

    def make_request(self, verb, resource, headers=None, data='',
                     expected_status=None, params=None):
        if headers is None:
            headers = {}
        response = AWSAuthConnection.make_request(
            self, verb, resource, headers=headers, data=data, params=params)
        body = json.loads(response.read().decode('utf-8'))
        if response.status == expected_status:
            return body
        else:
            error_type = response.getheader('x-amzn-ErrorType').split(':')[0]
            error_class = self._faults.get(error_type, self.ResponseError)
            raise error_class(response.status, response.reason, body)



# Copyright (c) 2014 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import boto
from boto.compat import json
from boto.connection import AWSQueryConnection
from boto.regioninfo import RegionInfo
from boto.exception import JSONResponseError
from boto.route53.domains import exceptions


class Route53DomainsConnection(AWSQueryConnection):
    """
    
    """
    APIVersion = "2014-05-15"
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "route53domains.us-east-1.amazonaws.com"
    ServiceName = "Route53Domains"
    TargetPrefix = "Route53Domains_v20140515"
    ResponseError = JSONResponseError

    _faults = {
        "DuplicateRequest": exceptions.DuplicateRequest,
        "DomainLimitExceeded": exceptions.DomainLimitExceeded,
        "InvalidInput": exceptions.InvalidInput,
        "OperationLimitExceeded": exceptions.OperationLimitExceeded,
        "UnsupportedTLD": exceptions.UnsupportedTLD,
        "TLDRulesViolation": exceptions.TLDRulesViolation,
    }


    def __init__(self, **kwargs):
        region = kwargs.pop('region', None)
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)

        if 'host' not in kwargs or kwargs['host'] is None:
            kwargs['host'] = region.endpoint

        super(Route53DomainsConnection, self).__init__(**kwargs)
        self.region = region

    def _required_auth_capability(self):
        return ['hmac-v4']

    def check_domain_availability(self, domain_name, idn_lang_code=None):
        """
        This operation checks the availability of one domain name. You
        can access this API without authenticating. Note that if the
        availability status of a domain is pending, you must submit
        another request to determine the availability of the domain
        name.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        :type idn_lang_code: string
        :param idn_lang_code: Reserved for future use.

        """
        params = {'DomainName': domain_name, }
        if idn_lang_code is not None:
            params['IdnLangCode'] = idn_lang_code
        return self.make_request(action='CheckDomainAvailability',
                                 body=json.dumps(params))

    def disable_domain_transfer_lock(self, domain_name):
        """
        This operation removes the transfer lock on the domain
        (specifically the `clientTransferProhibited` status) to allow
        domain transfers. We recommend you refrain from performing
        this action unless you intend to transfer the domain to a
        different registrar. Successful submission returns an
        operation ID that you can use to track the progress and
        completion of the action. If the request is not completed
        successfully, the domain registrant will be notified by email.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        """
        params = {'DomainName': domain_name, }
        return self.make_request(action='DisableDomainTransferLock',
                                 body=json.dumps(params))

    def enable_domain_transfer_lock(self, domain_name):
        """
        This operation sets the transfer lock on the domain
        (specifically the `clientTransferProhibited` status) to
        prevent domain transfers. Successful submission returns an
        operation ID that you can use to track the progress and
        completion of the action. If the request is not completed
        successfully, the domain registrant will be notified by email.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        """
        params = {'DomainName': domain_name, }
        return self.make_request(action='EnableDomainTransferLock',
                                 body=json.dumps(params))

    def get_domain_detail(self, domain_name):
        """
        This operation returns detailed information about the domain.
        The domain's contact information is also returned as part of
        the output.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        """
        params = {'DomainName': domain_name, }
        return self.make_request(action='GetDomainDetail',
                                 body=json.dumps(params))

    def get_operation_detail(self, operation_id):
        """
        This operation returns the current status of an operation that
        is not completed.

        :type operation_id: string
        :param operation_id: The identifier for the operation for which you
            want to get the status. Amazon Route 53 returned the identifier in
            the response to the original request.
        Type: String

        Default: None

        Required: Yes

        """
        params = {'OperationId': operation_id, }
        return self.make_request(action='GetOperationDetail',
                                 body=json.dumps(params))

    def list_domains(self, marker=None, max_items=None):
        """
        This operation returns all the domain names registered with
        Amazon Route 53 for the current AWS account.

        :type marker: string
        :param marker: For an initial request for a list of domains, omit this
            element. If the number of domains that are associated with the
            current AWS account is greater than the value that you specified
            for `MaxItems`, you can use `Marker` to return additional domains.
            Get the value of `NextPageMarker` from the previous response, and
            submit another request that includes the value of `NextPageMarker`
            in the `Marker` element.
        Type: String

        Default: None

        Constraints: The marker must match the value specified in the previous
            request.

        Required: No

        :type max_items: integer
        :param max_items: Number of domains to be returned.
        Type: Integer

        Default: 20

        Constraints: A numeral between 1 and 100.

        Required: No

        """
        params = {}
        if marker is not None:
            params['Marker'] = marker
        if max_items is not None:
            params['MaxItems'] = max_items
        return self.make_request(action='ListDomains',
                                 body=json.dumps(params))

    def list_operations(self, marker=None, max_items=None):
        """
        This operation returns the operation IDs of operations that
        are not yet complete.

        :type marker: string
        :param marker: For an initial request for a list of operations, omit
            this element. If the number of operations that are not yet complete
            is greater than the value that you specified for `MaxItems`, you
            can use `Marker` to return additional operations. Get the value of
            `NextPageMarker` from the previous response, and submit another
            request that includes the value of `NextPageMarker` in the `Marker`
            element.
        Type: String

        Default: None

        Required: No

        :type max_items: integer
        :param max_items: Number of domains to be returned.
        Type: Integer

        Default: 20

        Constraints: A value between 1 and 100.

        Required: No

        """
        params = {}
        if marker is not None:
            params['Marker'] = marker
        if max_items is not None:
            params['MaxItems'] = max_items
        return self.make_request(action='ListOperations',
                                 body=json.dumps(params))

    def register_domain(self, domain_name, duration_in_years, admin_contact,
                        registrant_contact, tech_contact, idn_lang_code=None,
                        auto_renew=None, privacy_protect_admin_contact=None,
                        privacy_protect_registrant_contact=None,
                        privacy_protect_tech_contact=None):
        """
        This operation registers a domain. Domains are registered by
        the AWS registrar partner, Gandi. For some top-level domains
        (TLDs), this operation requires extra parameters.

        When you register a domain, Amazon Route 53 does the
        following:


        + Creates a Amazon Route 53 hosted zone that has the same name
          as the domain. Amazon Route 53 assigns four name servers to
          your hosted zone and automatically updates your domain
          registration with the names of these name servers.
        + Enables autorenew, so your domain registration will renew
          automatically each year. We'll notify you in advance of the
          renewal date so you can choose whether to renew the
          registration.
        + Optionally enables privacy protection, so WHOIS queries
          return contact information for our registrar partner, Gandi,
          instead of the information you entered for registrant, admin,
          and tech contacts.
        + If registration is successful, returns an operation ID that
          you can use to track the progress and completion of the
          action. If the request is not completed successfully, the
          domain registrant is notified by email.
        + Charges your AWS account an amount based on the top-level
          domain. For more information, see `Amazon Route 53 Pricing`_.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        :type idn_lang_code: string
        :param idn_lang_code: Reserved for future use.

        :type duration_in_years: integer
        :param duration_in_years: The number of years the domain will be
            registered. Domains are registered for a minimum of one year. The
            maximum period depends on the top-level domain.
        Type: Integer

        Default: 1

        Valid values: Integer from 1 to 10

        Required: Yes

        :type auto_renew: boolean
        :param auto_renew: Indicates whether the domain will be automatically
            renewed ( `True`) or not ( `False`). Autorenewal only takes effect
            after the account is charged.
        Type: Boolean

        Valid values: `True` | `False`

        Default: `True`

        Required: No

        :type admin_contact: dict
        :param admin_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type registrant_contact: dict
        :param registrant_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type tech_contact: dict
        :param tech_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type privacy_protect_admin_contact: boolean
        :param privacy_protect_admin_contact: Whether you want to conceal
            contact information from WHOIS queries. If you specify true, WHOIS
            ("who is") queries will return contact information for our
            registrar partner, Gandi, instead of the contact information that
            you enter.
        Type: Boolean

        Default: `True`

        Valid values: `True` | `False`

        Required: No

        :type privacy_protect_registrant_contact: boolean
        :param privacy_protect_registrant_contact: Whether you want to conceal
            contact information from WHOIS queries. If you specify true, WHOIS
            ("who is") queries will return contact information for our
            registrar partner, Gandi, instead of the contact information that
            you enter.
        Type: Boolean

        Default: `True`

        Valid values: `True` | `False`

        Required: No

        :type privacy_protect_tech_contact: boolean
        :param privacy_protect_tech_contact: Whether you want to conceal
            contact information from WHOIS queries. If you specify true, WHOIS
            ("who is") queries will return contact information for our
            registrar partner, Gandi, instead of the contact information that
            you enter.
        Type: Boolean

        Default: `True`

        Valid values: `True` | `False`

        Required: No

        """
        params = {
            'DomainName': domain_name,
            'DurationInYears': duration_in_years,
            'AdminContact': admin_contact,
            'RegistrantContact': registrant_contact,
            'TechContact': tech_contact,
        }
        if idn_lang_code is not None:
            params['IdnLangCode'] = idn_lang_code
        if auto_renew is not None:
            params['AutoRenew'] = auto_renew
        if privacy_protect_admin_contact is not None:
            params['PrivacyProtectAdminContact'] = privacy_protect_admin_contact
        if privacy_protect_registrant_contact is not None:
            params['PrivacyProtectRegistrantContact'] = privacy_protect_registrant_contact
        if privacy_protect_tech_contact is not None:
            params['PrivacyProtectTechContact'] = privacy_protect_tech_contact
        return self.make_request(action='RegisterDomain',
                                 body=json.dumps(params))

    def retrieve_domain_auth_code(self, domain_name):
        """
        This operation returns the AuthCode for the domain. To
        transfer a domain to another registrar, you provide this value
        to the new registrar.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        """
        params = {'DomainName': domain_name, }
        return self.make_request(action='RetrieveDomainAuthCode',
                                 body=json.dumps(params))

    def transfer_domain(self, domain_name, duration_in_years, nameservers,
                        admin_contact, registrant_contact, tech_contact,
                        idn_lang_code=None, auth_code=None, auto_renew=None,
                        privacy_protect_admin_contact=None,
                        privacy_protect_registrant_contact=None,
                        privacy_protect_tech_contact=None):
        """
        This operation transfers a domain from another registrar to
        Amazon Route 53. Domains are registered by the AWS registrar,
        Gandi upon transfer.

        To transfer a domain, you need to meet all the domain transfer
        criteria, including the following:


        + You must supply nameservers to transfer a domain.
        + You must disable the domain transfer lock (if any) before
          transferring the domain.
        + A minimum of 60 days must have elapsed since the domain's
          registration or last transfer.


        We recommend you use the Amazon Route 53 as the DNS service
        for your domain. You can create a hosted zone in Amazon Route
        53 for your current domain before transferring your domain.

        Note that upon transfer, the domain duration is extended for a
        year if not otherwise specified. Autorenew is enabled by
        default.

        If the transfer is successful, this method returns an
        operation ID that you can use to track the progress and
        completion of the action. If the request is not completed
        successfully, the domain registrant will be notified by email.

        Transferring domains charges your AWS account an amount based
        on the top-level domain. For more information, see `Amazon
        Route 53 Pricing`_.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        :type idn_lang_code: string
        :param idn_lang_code: Reserved for future use.

        :type duration_in_years: integer
        :param duration_in_years: The number of years the domain will be
            registered. Domains are registered for a minimum of one year. The
            maximum period depends on the top-level domain.
        Type: Integer

        Default: 1

        Valid values: Integer from 1 to 10

        Required: Yes

        :type nameservers: list
        :param nameservers: Contains details for the host and glue IP
            addresses.
        Type: Complex

        Children: `GlueIps`, `Name`

        :type auth_code: string
        :param auth_code: The authorization code for the domain. You get this
            value from the current registrar.
        Type: String

        Required: Yes

        :type auto_renew: boolean
        :param auto_renew: Indicates whether the domain will be automatically
            renewed (true) or not (false). Autorenewal only takes effect after
            the account is charged.
        Type: Boolean

        Valid values: `True` | `False`

        Default: true

        Required: No

        :type admin_contact: dict
        :param admin_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type registrant_contact: dict
        :param registrant_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type tech_contact: dict
        :param tech_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type privacy_protect_admin_contact: boolean
        :param privacy_protect_admin_contact: Whether you want to conceal
            contact information from WHOIS queries. If you specify true, WHOIS
            ("who is") queries will return contact information for our
            registrar partner, Gandi, instead of the contact information that
            you enter.
        Type: Boolean

        Default: `True`

        Valid values: `True` | `False`

        Required: No

        :type privacy_protect_registrant_contact: boolean
        :param privacy_protect_registrant_contact: Whether you want to conceal
            contact information from WHOIS queries. If you specify true, WHOIS
            ("who is") queries will return contact information for our
            registrar partner, Gandi, instead of the contact information that
            you enter.
        Type: Boolean

        Default: `True`

        Valid values: `True` | `False`

        Required: No

        :type privacy_protect_tech_contact: boolean
        :param privacy_protect_tech_contact: Whether you want to conceal
            contact information from WHOIS queries. If you specify true, WHOIS
            ("who is") queries will return contact information for our
            registrar partner, Gandi, instead of the contact information that
            you enter.
        Type: Boolean

        Default: `True`

        Valid values: `True` | `False`

        Required: No

        """
        params = {
            'DomainName': domain_name,
            'DurationInYears': duration_in_years,
            'Nameservers': nameservers,
            'AdminContact': admin_contact,
            'RegistrantContact': registrant_contact,
            'TechContact': tech_contact,
        }
        if idn_lang_code is not None:
            params['IdnLangCode'] = idn_lang_code
        if auth_code is not None:
            params['AuthCode'] = auth_code
        if auto_renew is not None:
            params['AutoRenew'] = auto_renew
        if privacy_protect_admin_contact is not None:
            params['PrivacyProtectAdminContact'] = privacy_protect_admin_contact
        if privacy_protect_registrant_contact is not None:
            params['PrivacyProtectRegistrantContact'] = privacy_protect_registrant_contact
        if privacy_protect_tech_contact is not None:
            params['PrivacyProtectTechContact'] = privacy_protect_tech_contact
        return self.make_request(action='TransferDomain',
                                 body=json.dumps(params))

    def update_domain_contact(self, domain_name, admin_contact=None,
                              registrant_contact=None, tech_contact=None):
        """
        This operation updates the contact information for a
        particular domain. Information for at least one contact
        (registrant, administrator, or technical) must be supplied for
        update.

        If the update is successful, this method returns an operation
        ID that you can use to track the progress and completion of
        the action. If the request is not completed successfully, the
        domain registrant will be notified by email.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        :type admin_contact: dict
        :param admin_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type registrant_contact: dict
        :param registrant_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        :type tech_contact: dict
        :param tech_contact: Provides detailed contact information.
        Type: Complex

        Children: `FirstName`, `MiddleName`, `LastName`, `ContactType`,
            `OrganizationName`, `AddressLine1`, `AddressLine2`, `City`,
            `State`, `CountryCode`, `ZipCode`, `PhoneNumber`, `Email`, `Fax`,
            `ExtraParams`

        Required: Yes

        """
        params = {'DomainName': domain_name, }
        if admin_contact is not None:
            params['AdminContact'] = admin_contact
        if registrant_contact is not None:
            params['RegistrantContact'] = registrant_contact
        if tech_contact is not None:
            params['TechContact'] = tech_contact
        return self.make_request(action='UpdateDomainContact',
                                 body=json.dumps(params))

    def update_domain_contact_privacy(self, domain_name, admin_privacy=None,
                                      registrant_privacy=None,
                                      tech_privacy=None):
        """
        This operation updates the specified domain contact's privacy
        setting. When the privacy option is enabled, personal
        information such as postal or email address is hidden from the
        results of a public WHOIS query. The privacy services are
        provided by the AWS registrar, Gandi. For more information,
        see the `Gandi privacy features`_.

        This operation only affects the privacy of the specified
        contact type (registrant, administrator, or tech). Successful
        acceptance returns an operation ID that you can use with
        GetOperationDetail to track the progress and completion of the
        action. If the request is not completed successfully, the
        domain registrant will be notified by email.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        :type admin_privacy: boolean
        :param admin_privacy: Whether you want to conceal contact information
            from WHOIS queries. If you specify true, WHOIS ("who is") queries
            will return contact information for our registrar partner, Gandi,
            instead of the contact information that you enter.
        Type: Boolean

        Default: None

        Valid values: `True` | `False`

        Required: No

        :type registrant_privacy: boolean
        :param registrant_privacy: Whether you want to conceal contact
            information from WHOIS queries. If you specify true, WHOIS ("who
            is") queries will return contact information for our registrar
            partner, Gandi, instead of the contact information that you enter.
        Type: Boolean

        Default: None

        Valid values: `True` | `False`

        Required: No

        :type tech_privacy: boolean
        :param tech_privacy: Whether you want to conceal contact information
            from WHOIS queries. If you specify true, WHOIS ("who is") queries
            will return contact information for our registrar partner, Gandi,
            instead of the contact information that you enter.
        Type: Boolean

        Default: None

        Valid values: `True` | `False`

        Required: No

        """
        params = {'DomainName': domain_name, }
        if admin_privacy is not None:
            params['AdminPrivacy'] = admin_privacy
        if registrant_privacy is not None:
            params['RegistrantPrivacy'] = registrant_privacy
        if tech_privacy is not None:
            params['TechPrivacy'] = tech_privacy
        return self.make_request(action='UpdateDomainContactPrivacy',
                                 body=json.dumps(params))

    def update_domain_nameservers(self, domain_name, nameservers):
        """
        This operation replaces the current set of name servers for
        the domain with the specified set of name servers. If you use
        Amazon Route 53 as your DNS service, specify the four name
        servers in the delegation set for the hosted zone for the
        domain.

        If successful, this operation returns an operation ID that you
        can use to track the progress and completion of the action. If
        the request is not completed successfully, the domain
        registrant will be notified by email.

        :type domain_name: string
        :param domain_name: The name of a domain.
        Type: String

        Default: None

        Constraints: The domain name can contain only the letters a through z,
            the numbers 0 through 9, and hyphen (-). Internationalized Domain
            Names are not supported.

        Required: Yes

        :type nameservers: list
        :param nameservers: A list of new name servers for the domain.
        Type: Complex

        Children: `Name`, `GlueIps`

        Required: Yes

        """
        params = {
            'DomainName': domain_name,
            'Nameservers': nameservers,
        }
        return self.make_request(action='UpdateDomainNameservers',
                                 body=json.dumps(params))

    def make_request(self, action, body):
        headers = {
            'X-Amz-Target': '%s.%s' % (self.TargetPrefix, action),
            'Host': self.region.endpoint,
            'Content-Type': 'application/x-amz-json-1.1',
            'Content-Length': str(len(body)),
        }
        http_request = self.build_base_http_request(
            method='POST', path='/', auth_path='/', params={},
            headers=headers, data=body)
        response = self._mexe(http_request, sender=None,
                              override_num_retries=10)
        response_body = response.read().decode('utf-8')
        boto.log.debug(response_body)
        if response.status == 200:
            if response_body:
                return json.loads(response_body)
        else:
            json_body = json.loads(response_body)
            fault_name = json_body.get('__type', None)
            exception_class = self._faults.get(fault_name, self.ResponseError)
            raise exception_class(response.status, response.reason,
                                  body=json_body)




from boto.manage.server import Server
from boto.manage.volume import Volume
import time

print('--> Creating New Volume')
volume = Volume.create()
print(volume)

print('--> Creating New Server')
server_list = Server.create()
server = server_list[0]
print(server)

print('----> Waiting for Server to start up')
while server.status != 'running':
    print('*')
    time.sleep(10)
print('----> Server is running')

print('--> Run "df -k" on Server')
status = server.run('df -k')
print(status[1])

print('--> Now run volume.make_ready to make the volume ready to use on server')
volume.make_ready(server)

print('--> Run "df -k" on Server')
status = server.run('df -k')
print(status[1])

print('--> Do an "ls -al" on the new filesystem')
status = server.run('ls -al %s' % volume.mount_point)
print(status[1])




# Copyright (c) 2009-2010 Mitch Garnaat http://garnaat.org/
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

"""
Represents a Virtual Private Cloud.
"""

from boto.ec2.ec2object import TaggedEC2Object

class VPC(TaggedEC2Object):

    def __init__(self, connection=None):
        """
        Represents a VPC.

        :ivar id: The unique ID of the VPC.
        :ivar dhcp_options_id: The ID of the set of DHCP options you've associated with the VPC
                                (or default if the default options are associated with the VPC).
        :ivar state: The current state of the VPC.
        :ivar cidr_block: The CIDR block for the VPC.
        :ivar is_default: Indicates whether the VPC is the default VPC.
        :ivar instance_tenancy: The allowed tenancy of instances launched into the VPC.
        :ivar classic_link_enabled: Indicates whether ClassicLink is enabled.
        """
        super(VPC, self).__init__(connection)
        self.id = None
        self.dhcp_options_id = None
        self.state = None
        self.cidr_block = None
        self.is_default = None
        self.instance_tenancy = None
        self.classic_link_enabled = None

    def __repr__(self):
        return 'VPC:%s' % self.id

    def endElement(self, name, value, connection):
        if name == 'vpcId':
            self.id = value
        elif name == 'dhcpOptionsId':
            self.dhcp_options_id = value
        elif name == 'state':
            self.state = value
        elif name == 'cidrBlock':
            self.cidr_block = value
        elif name == 'isDefault':
            self.is_default = True if value == 'true' else False
        elif name == 'instanceTenancy':
            self.instance_tenancy = value
        elif name == 'classicLinkEnabled':
            self.classic_link_enabled = value
        else:
            setattr(self, name, value)

    def delete(self):
        return self.connection.delete_vpc(self.id)

    def _update(self, updated):
        self.__dict__.update(updated.__dict__)

    def _get_status_then_update_vpc(self, get_status_method, validate=False,
                                    dry_run=False):
        vpc_list = get_status_method(
            [self.id],
            dry_run=dry_run
        )
        if len(vpc_list):
            updated_vpc = vpc_list[0]
            self._update(updated_vpc)
        elif validate:
            raise ValueError('%s is not a valid VPC ID' % (self.id,))

    def update(self, validate=False, dry_run=False):
        self._get_status_then_update_vpc(
            self.connection.get_all_vpcs,
            validate=validate,
            dry_run=dry_run
        )
        return self.state

    def update_classic_link_enabled(self, validate=False, dry_run=False):
        """
        Updates instance's classic_link_enabled attribute

        :rtype: bool
        :return: self.classic_link_enabled after update has occurred.
        """
        self._get_status_then_update_vpc(
            self.connection.get_all_classic_link_vpcs,
            validate=validate,
            dry_run=dry_run
        )
        return self.classic_link_enabled

    def disable_classic_link(self, dry_run=False):
        """
        Disables  ClassicLink  for  a VPC. You cannot disable ClassicLink for a
        VPC that has EC2-Classic instances linked to it.

        :type dry_run: bool
        :param dry_run: Set to True if the operation should not actually run.

        :rtype: bool
        :return: True if successful
        """
        return self.connection.disable_vpc_classic_link(self.id,
                                                        dry_run=dry_run)

    def enable_classic_link(self, dry_run=False):
        """
        Enables a VPC for ClassicLink. You can then link EC2-Classic instances
        to your ClassicLink-enabled VPC to allow communication over private IP
        addresses. You cannot enable your VPC for ClassicLink if any of your
        VPC's route tables have existing routes for address ranges within the
        10.0.0.0/8 IP address range, excluding local routes for VPCs in the
        10.0.0.0/16 and 10.1.0.0/16 IP address ranges.

        :type dry_run: bool
        :param dry_run: Set to True if the operation should not actually run.

        :rtype: bool
        :return: True if successful
        """
        return self.connection.enable_vpc_classic_link(self.id,
                                                       dry_run=dry_run)

    def attach_classic_instance(self, instance_id, groups, dry_run=False):
        """
        Links  an EC2-Classic instance to a ClassicLink-enabled VPC through one
        or more of the VPC's security groups. You cannot link an EC2-Classic
        instance to more than one VPC at a time. You can only link an instance
        that's in the running state. An instance is automatically unlinked from
        a VPC when it's stopped. You can link it to the VPC again when you
        restart it.

        After you've linked an instance, you cannot  change  the VPC security
        groups  that are associated with it. To change the security groups, you
        must first unlink the instance, and then link it again.

        Linking your instance to a VPC is sometimes referred  to  as  attaching
        your instance.

        :type intance_id: str
        :param instance_is: The ID of a ClassicLink-enabled VPC.

        :tye groups: list
        :param groups: The ID of one or more of the VPC's security groups.
            You cannot specify security groups from a different VPC. The
            members of the list can be
            :class:`boto.ec2.securitygroup.SecurityGroup` objects or
            strings of the id's of the security groups.

        :type dry_run: bool
        :param dry_run: Set to True if the operation should not actually run.

        :rtype: bool
        :return: True if successful
        """
        return self.connection.attach_classic_link_vpc(
            vpc_id=self.id,
            instance_id=instance_id,
            groups=groups,
            dry_run=dry_run
        )

    def detach_classic_instance(self, instance_id, dry_run=False):
        """
        Unlinks a linked EC2-Classic instance from a VPC. After the instance
        has been unlinked, the VPC security groups are no longer associated
        with it. An instance is automatically unlinked from a VPC when
        it's stopped.

        :type intance_id: str
        :param instance_is: The ID of the VPC to which the instance is linked.

        :type dry_run: bool
        :param dry_run: Set to True if the operation should not actually run.

        :rtype: bool
        :return: True if successful
        """
        return self.connection.detach_classic_link_vpc(
            vpc_id=self.id,
            instance_id=instance_id,
            dry_run=dry_run
        )




import argparse
import errno
import os

import boto
from boto.compat import json
from boto.compat import six


DESCRIPTION = """Dump the contents of one or more DynamoDB tables to the local filesystem.

Each table is dumped into two files:
  - {table_name}.metadata stores the table's name, schema and provisioned
    throughput.
  - {table_name}.data stores the table's actual contents.

Both files are created in the current directory. To write them somewhere else,
use the --out-dir parameter (the target directory will be created if needed).
"""


def dump_table(table, out_dir):
    metadata_file = os.path.join(out_dir, "%s.metadata" % table.name)
    data_file = os.path.join(out_dir, "%s.data" % table.name)

    with open(metadata_file, "w") as metadata_fd:
        json.dump(
            {
                "name": table.name,
                "schema": table.schema.dict,
                "read_units": table.read_units,
                "write_units": table.write_units,
            },
            metadata_fd
        )

    with open(data_file, "w") as data_fd:
        for item in table.scan():
            # JSON can't serialize sets -- convert those to lists.
            data = {}
            for k, v in six.iteritems(item):
                if isinstance(v, (set, frozenset)):
                    data[k] = list(v)
                else:
                    data[k] = v

            data_fd.write(json.dumps(data))
            data_fd.write("\n")


def dynamodb_dump(tables, out_dir):
    try:
        os.makedirs(out_dir)
    except OSError as e:
        # We don't care if the dir already exists.
        if e.errno != errno.EEXIST:
            raise

    conn = boto.connect_dynamodb()
    for t in tables:
        dump_table(conn.get_table(t), out_dir)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="dynamodb_dump",
        description=DESCRIPTION
    )
    parser.add_argument("--out-dir", default=".")
    parser.add_argument("tables", metavar="TABLES", nargs="+")

    namespace = parser.parse_args()

    dynamodb_dump(namespace.tables, namespace.out_dir)



# Copyright (c) 2006-2009 Mitch Garnaat http://garnaat.org/
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
"""
The cmdshell module uses the paramiko package to create SSH connections
to the servers that are represented by instance objects. The module has
functions for running commands, managing files, and opening interactive
shell sessions over those connections.
"""
from boto.mashups.interactive import interactive_shell
import boto
import os
import time
import shutil
import paramiko
import socket
import subprocess

from boto.compat import StringIO

class SSHClient(object):
    """
    This class creates a paramiko.SSHClient() object that represents
    a session with an SSH server. You can use the SSHClient object to send
    commands to the remote host and manipulate files on the remote host. 
    
    :ivar server: A Server object or FakeServer object.
    :ivar host_key_file: The path to the user's .ssh key files.
    :ivar uname: The username for the SSH connection. Default = 'root'.
    :ivar timeout: The optional timeout variable for the TCP connection.
    :ivar ssh_pwd: An optional password to use for authentication or for
                    unlocking the private key.
    """
    def __init__(self, server,
                 host_key_file='~/.ssh/known_hosts',
                 uname='root', timeout=None, ssh_pwd=None):
        self.server = server
        self.host_key_file = host_key_file
        self.uname = uname
        self._timeout = timeout
        self._pkey = paramiko.RSAKey.from_private_key_file(server.ssh_key_file,
                                                           password=ssh_pwd)
        self._ssh_client = paramiko.SSHClient()
        self._ssh_client.load_system_host_keys()
        self._ssh_client.load_host_keys(os.path.expanduser(host_key_file))
        self._ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.connect()

    def connect(self, num_retries=5):
        """
        Connect to an SSH server and authenticate with it.
        
        :type num_retries: int
        :param num_retries: The maximum number of connection attempts.
        """
        retry = 0
        while retry < num_retries:
            try:
                self._ssh_client.connect(self.server.hostname,
                                         username=self.uname,
                                         pkey=self._pkey,
                                         timeout=self._timeout)
                return
            except socket.error as xxx_todo_changeme:
                (value, message) = xxx_todo_changeme.args
                if value in (51, 61, 111):
                    print('SSH Connection refused, will retry in 5 seconds')
                    time.sleep(5)
                    retry += 1
                else:
                    raise
            except paramiko.BadHostKeyException:
                print("%s has an entry in ~/.ssh/known_hosts and it doesn't match" % self.server.hostname)
                print('Edit that file to remove the entry and then hit return to try again')
                raw_input('Hit Enter when ready')
                retry += 1
            except EOFError:
                print('Unexpected Error from SSH Connection, retry in 5 seconds')
                time.sleep(5)
                retry += 1
        print('Could not establish SSH connection')

    def open_sftp(self):
        """
        Open an SFTP session on the SSH server.
        
        :rtype: :class:`paramiko.sftp_client.SFTPClient`
        :return: An SFTP client object.
        """
        return self._ssh_client.open_sftp()

    def get_file(self, src, dst):
        """
        Open an SFTP session on the remote host, and copy a file from
        the remote host to the specified path on the local host.
        
        :type src: string
        :param src: The path to the target file on the remote host.
        
        :type dst: string
        :param dst: The path on your local host where you want to
                    store the file.
        """
        sftp_client = self.open_sftp()
        sftp_client.get(src, dst)

    def put_file(self, src, dst):
        """
        Open an SFTP session on the remote host, and copy a file from
        the local host to the specified path on the remote host.
        
        :type src: string
        :param src: The path to the target file on your local host.
        
        :type dst: string
        :param dst: The path on the remote host where you want to store
                    the file.
        """
        sftp_client = self.open_sftp()
        sftp_client.put(src, dst)

    def open(self, filename, mode='r', bufsize=-1):
        """
        Open an SFTP session to the remote host, and open a file on
        that host.
        
        :type filename: string
        :param filename: The path to the file on the remote host.
        
        :type mode: string
        :param mode: The file interaction mode.
        
        :type bufsize: integer
        :param bufsize: The file buffer size.
        
        :rtype: :class:`paramiko.sftp_file.SFTPFile`
        :return: A paramiko proxy object for a file on the remote server.
        """
        sftp_client = self.open_sftp()
        return sftp_client.open(filename, mode, bufsize)

    def listdir(self, path):
        """
        List all of the files and subdirectories at the specified path
        on the remote host.
        
        :type path: string
        :param path: The base path from which to obtain the list.
          
        :rtype: list
        :return: A list of files and subdirectories at the specified path.
        """
        sftp_client = self.open_sftp()
        return sftp_client.listdir(path)

    def isdir(self, path):
        """
        Check the specified path on the remote host to determine if
        it is a directory.
        
        :type path: string
        :param path: The path to the directory that you want to check.
        
        :rtype: integer
        :return: If the path is a directory, the function returns 1.
                If the path is a file or an invalid path, the function
                returns 0.
        """
        status = self.run('[ -d %s ] || echo "FALSE"' % path)
        if status[1].startswith('FALSE'):
            return 0
        return 1

    def exists(self, path):
        """
        Check the remote host for the specified path, or a file
        at the specified path. This function returns 1 if the
        path or the file exist on the remote host, and returns 0 if
        the path or the file does not exist on the remote host.
        
        :type path: string
        :param path: The path to the directory or file that you want to check.
        
        :rtype: integer
        :return: If the path or the file exist, the function returns 1.
                If the path or the file do not exist on the remote host,
                the function returns 0.
        """
        
        status = self.run('[ -a %s ] || echo "FALSE"' % path)
        if status[1].startswith('FALSE'):
            return 0
        return 1

    def shell(self):
        """
        Start an interactive shell session with the remote host.
        """
        channel = self._ssh_client.invoke_shell()
        interactive_shell(channel)

    def run(self, command):
        """
        Run a command on the remote host.
        
        :type command: string
        :param command: The command that you want to send to the remote host.

        :rtype: tuple
        :return: This function returns a tuple that contains an integer status,
                the stdout from the command, and the stderr from the command.

        """
        boto.log.debug('running:%s on %s' % (command, self.server.instance_id))
        status = 0
        try:
            t = self._ssh_client.exec_command(command)
        except paramiko.SSHException:
            status = 1
        std_out = t[1].read()
        std_err = t[2].read()
        t[0].close()
        t[1].close()
        t[2].close()
        boto.log.debug('stdout: %s' % std_out)
        boto.log.debug('stderr: %s' % std_err)
        return (status, std_out, std_err)

    def run_pty(self, command):
        """
        Request a pseudo-terminal from a server, and execute a command on that
        server.

        :type command: string
        :param command: The command that you want to run on the remote host.
        
        :rtype: :class:`paramiko.channel.Channel`
        :return: An open channel object.
        """
        boto.log.debug('running:%s on %s' % (command, self.server.instance_id))
        channel = self._ssh_client.get_transport().open_session()
        channel.get_pty()
        channel.exec_command(command)
        return channel

    def close(self):
        """
        Close an SSH session and any open channels that are tied to it.
        """
        transport = self._ssh_client.get_transport()
        transport.close()
        self.server.reset_cmdshell()

class LocalClient(object):
    """
    :ivar server: A Server object or FakeServer object.
    :ivar host_key_file: The path to the user's .ssh key files.
    :ivar uname: The username for the SSH connection. Default = 'root'.
    """
    def __init__(self, server, host_key_file=None, uname='root'):
        self.server = server
        self.host_key_file = host_key_file
        self.uname = uname

    def get_file(self, src, dst):
        """
        Copy a file from one directory to another.
        """
        shutil.copyfile(src, dst)

    def put_file(self, src, dst):
        """
        Copy a file from one directory to another.
        """
        shutil.copyfile(src, dst)

    def listdir(self, path):
        """
        List all of the files and subdirectories at the specified path.
        
        :rtype: list
        :return: Return a list containing the names of the entries
                in the directory given by path.
        """
        return os.listdir(path)

    def isdir(self, path):
        """
        Check the specified path to determine if it is a directory.
        
        :rtype: boolean
        :return: Returns True if the path is an existing directory.
        """
        return os.path.isdir(path)

    def exists(self, path):
        """
        Check for the specified path, or check a file at the specified path.
        
        :rtype: boolean
        :return: If the path or the file exist, the function returns True.
        """
        return os.path.exists(path)

    def shell(self):
        raise NotImplementedError('shell not supported with LocalClient')

    def run(self):
        """
        Open a subprocess and run a command on the local host.
        
        :rtype: tuple
        :return: This function returns a tuple that contains an integer status
                and a string with the combined stdout and stderr output.
        """
        boto.log.info('running:%s' % self.command)
        log_fp = StringIO()
        process = subprocess.Popen(self.command, shell=True, stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        while process.poll() is None:
            time.sleep(1)
            t = process.communicate()
            log_fp.write(t[0])
            log_fp.write(t[1])
        boto.log.info(log_fp.getvalue())
        boto.log.info('output: %s' % log_fp.getvalue())
        return (process.returncode, log_fp.getvalue())

    def close(self):
        pass

class FakeServer(object):
    """
    This object has a subset of the variables that are normally in a
    :class:`boto.manage.server.Server` object. You can use this FakeServer
    object to create a :class:`boto.manage.SSHClient` object if you
    don't have a real Server object.
    
    :ivar instance: A boto Instance object.
    :ivar ssh_key_file: The path to the SSH key file.
    """
    def __init__(self, instance, ssh_key_file):
        self.instance = instance
        self.ssh_key_file = ssh_key_file
        self.hostname = instance.dns_name
        self.instance_id = self.instance.id

def start(server):
    """
    Connect to the specified server.

    :return: If the server is local, the function returns a 
            :class:`boto.manage.cmdshell.LocalClient` object.
            If the server is remote, the function returns a
            :class:`boto.manage.cmdshell.SSHClient` object.
    """
    instance_id = boto.config.get('Instance', 'instance-id', None)
    if instance_id == server.instance_id:
        return LocalClient(server)
    else:
        return SSHClient(server)

def sshclient_from_instance(instance, ssh_key_file,
                            host_key_file='~/.ssh/known_hosts',
                            user_name='root', ssh_pwd=None):
    """
    Create and return an SSHClient object given an
    instance object.

    :type instance: :class`boto.ec2.instance.Instance` object
    :param instance: The instance object.

    :type ssh_key_file: string
    :param ssh_key_file: A path to the private key file that is 
                        used to log into the instance.

    :type host_key_file: string
    :param host_key_file: A path to the known_hosts file used
                          by the SSH client.
                          Defaults to ~/.ssh/known_hosts
    :type user_name: string
    :param user_name: The username to use when logging into
                      the instance.  Defaults to root.

    :type ssh_pwd: string
    :param ssh_pwd: The passphrase, if any, associated with
                    private key.
    """
    s = FakeServer(instance, ssh_key_file)
    return SSHClient(s, host_key_file, user_name, ssh_pwd)



# Copyright (c) 2014 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import boto
from boto.compat import json
from boto.connection import AWSQueryConnection
from boto.regioninfo import RegionInfo
from boto.exception import JSONResponseError
from boto.cloudsearch2 import exceptions


class CloudSearchConnection(AWSQueryConnection):
    """
    Amazon CloudSearch Configuration Service
    You use the Amazon CloudSearch configuration service to create,
    configure, and manage search domains. Configuration service
    requests are submitted using the AWS Query protocol. AWS Query
    requests are HTTP or HTTPS requests submitted via HTTP GET or POST
    with a query parameter named Action.

    The endpoint for configuration service requests is region-
    specific: cloudsearch. region .amazonaws.com. For example,
    cloudsearch.us-east-1.amazonaws.com. For a current list of
    supported regions and endpoints, see `Regions and Endpoints`_.
    """
    APIVersion = "2013-01-01"
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "cloudsearch.us-east-1.amazonaws.com"
    ResponseError = JSONResponseError

    _faults = {
        "InvalidTypeException": exceptions.InvalidTypeException,
        "LimitExceededException": exceptions.LimitExceededException,
        "InternalException": exceptions.InternalException,
        "DisabledOperationException": exceptions.DisabledOperationException,
        "ResourceNotFoundException": exceptions.ResourceNotFoundException,
        "BaseException": exceptions.BaseException,
    }

    def __init__(self, **kwargs):
        region = kwargs.pop('region', None)
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)

        if 'host' not in kwargs or kwargs['host'] is None:
            kwargs['host'] = region.endpoint

        sign_request = kwargs.pop('sign_request', False)
        self.sign_request = sign_request

        super(CloudSearchConnection, self).__init__(**kwargs)
        self.region = region

    def _required_auth_capability(self):
        return ['hmac-v4']

    def build_suggesters(self, domain_name):
        """
        Indexes the search suggestions.

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        """
        params = {'DomainName': domain_name, }
        return self._make_request(
            action='BuildSuggesters',
            verb='POST',
            path='/', params=params)

    def create_domain(self, domain_name):
        """
        Creates a new search domain. For more information, see
        `Creating a Search Domain`_ in the Amazon CloudSearch
        Developer Guide .

        :type domain_name: string
        :param domain_name: A name for the domain you are creating. Allowed
            characters are a-z (lower-case letters), 0-9, and hyphen (-).
            Domain names must start with a letter or number and be at least 3
            and no more than 28 characters long.

        """
        params = {'DomainName': domain_name, }
        return self._make_request(
            action='CreateDomain',
            verb='POST',
            path='/', params=params)

    def define_analysis_scheme(self, domain_name, analysis_scheme):
        """
        Configures an analysis scheme that can be applied to a `text`
        or `text-array` field to define language-specific text
        processing options. For more information, see `Configuring
        Analysis Schemes`_ in the Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type analysis_scheme: dict
        :param analysis_scheme: Configuration information for an analysis
            scheme. Each analysis scheme has a unique name and specifies the
            language of the text to be processed. The following options can be
            configured for an analysis scheme: `Synonyms`, `Stopwords`,
            `StemmingDictionary`, and `AlgorithmicStemming`.

        """
        params = {'DomainName': domain_name, }
        self.build_complex_param(params, 'AnalysisScheme',
                                 analysis_scheme)
        return self._make_request(
            action='DefineAnalysisScheme',
            verb='POST',
            path='/', params=params)

    def define_expression(self, domain_name, expression):
        """
        Configures an `Expression` for the search domain. Used to
        create new expressions and modify existing ones. If the
        expression exists, the new configuration replaces the old one.
        For more information, see `Configuring Expressions`_ in the
        Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type expression: dict
        :param expression: A named expression that can be evaluated at search
            time. Can be used to sort the search results, define other
            expressions, or return computed information in the search results.

        """
        params = {'DomainName': domain_name, }
        self.build_complex_param(params, 'Expression',
                                 expression)
        return self._make_request(
            action='DefineExpression',
            verb='POST',
            path='/', params=params)

    def define_index_field(self, domain_name, index_field):
        """
        Configures an `IndexField` for the search domain. Used to
        create new fields and modify existing ones. You must specify
        the name of the domain you are configuring and an index field
        configuration. The index field configuration specifies a
        unique name, the index field type, and the options you want to
        configure for the field. The options you can specify depend on
        the `IndexFieldType`. If the field exists, the new
        configuration replaces the old one. For more information, see
        `Configuring Index Fields`_ in the Amazon CloudSearch
        Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type index_field: dict
        :param index_field: The index field and field options you want to
            configure.

        """
        params = {'DomainName': domain_name, }
        self.build_complex_param(params, 'IndexField',
                                 index_field)
        return self._make_request(
            action='DefineIndexField',
            verb='POST',
            path='/', params=params)

    def define_suggester(self, domain_name, suggester):
        """
        Configures a suggester for a domain. A suggester enables you
        to display possible matches before users finish typing their
        queries. When you configure a suggester, you must specify the
        name of the text field you want to search for possible matches
        and a unique name for the suggester. For more information, see
        `Getting Search Suggestions`_ in the Amazon CloudSearch
        Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type suggester: dict
        :param suggester: Configuration information for a search suggester.
            Each suggester has a unique name and specifies the text field you
            want to use for suggestions. The following options can be
            configured for a suggester: `FuzzyMatching`, `SortExpression`.

        """
        params = {'DomainName': domain_name, }
        self.build_complex_param(params, 'Suggester',
                                 suggester)
        return self._make_request(
            action='DefineSuggester',
            verb='POST',
            path='/', params=params)

    def delete_analysis_scheme(self, domain_name, analysis_scheme_name):
        """
        Deletes an analysis scheme. For more information, see
        `Configuring Analysis Schemes`_ in the Amazon CloudSearch
        Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type analysis_scheme_name: string
        :param analysis_scheme_name: The name of the analysis scheme you want
            to delete.

        """
        params = {
            'DomainName': domain_name,
            'AnalysisSchemeName': analysis_scheme_name,
        }
        return self._make_request(
            action='DeleteAnalysisScheme',
            verb='POST',
            path='/', params=params)

    def delete_domain(self, domain_name):
        """
        Permanently deletes a search domain and all of its data. Once
        a domain has been deleted, it cannot be recovered. For more
        information, see `Deleting a Search Domain`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to permanently
            delete.

        """
        params = {'DomainName': domain_name, }
        return self._make_request(
            action='DeleteDomain',
            verb='POST',
            path='/', params=params)

    def delete_expression(self, domain_name, expression_name):
        """
        Removes an `Expression` from the search domain. For more
        information, see `Configuring Expressions`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type expression_name: string
        :param expression_name: The name of the `Expression` to delete.

        """
        params = {
            'DomainName': domain_name,
            'ExpressionName': expression_name,
        }
        return self._make_request(
            action='DeleteExpression',
            verb='POST',
            path='/', params=params)

    def delete_index_field(self, domain_name, index_field_name):
        """
        Removes an `IndexField` from the search domain. For more
        information, see `Configuring Index Fields`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type index_field_name: string
        :param index_field_name: The name of the index field your want to
            remove from the domain's indexing options.

        """
        params = {
            'DomainName': domain_name,
            'IndexFieldName': index_field_name,
        }
        return self._make_request(
            action='DeleteIndexField',
            verb='POST',
            path='/', params=params)

    def delete_suggester(self, domain_name, suggester_name):
        """
        Deletes a suggester. For more information, see `Getting Search
        Suggestions`_ in the Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type suggester_name: string
        :param suggester_name: Specifies the name of the suggester you want to
            delete.

        """
        params = {
            'DomainName': domain_name,
            'SuggesterName': suggester_name,
        }
        return self._make_request(
            action='DeleteSuggester',
            verb='POST',
            path='/', params=params)

    def describe_analysis_schemes(self, domain_name,
                                  analysis_scheme_names=None, deployed=None):
        """
        Gets the analysis schemes configured for a domain. An analysis
        scheme defines language-specific text processing options for a
        `text` field. Can be limited to specific analysis schemes by
        name. By default, shows all analysis schemes and includes any
        pending changes to the configuration. Set the `Deployed`
        option to `True` to show the active configuration and exclude
        pending changes. For more information, see `Configuring
        Analysis Schemes`_ in the Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to describe.

        :type analysis_scheme_names: list
        :param analysis_scheme_names: The analysis schemes you want to
            describe.

        :type deployed: boolean
        :param deployed: Whether to display the deployed configuration (
            `True`) or include any pending changes ( `False`). Defaults to
            `False`.

        """
        params = {'DomainName': domain_name, }
        if analysis_scheme_names is not None:
            self.build_list_params(params,
                                   analysis_scheme_names,
                                   'AnalysisSchemeNames.member')
        if deployed is not None:
            params['Deployed'] = str(
                deployed).lower()
        return self._make_request(
            action='DescribeAnalysisSchemes',
            verb='POST',
            path='/', params=params)

    def describe_availability_options(self, domain_name, deployed=None):
        """
        Gets the availability options configured for a domain. By
        default, shows the configuration with any pending changes. Set
        the `Deployed` option to `True` to show the active
        configuration and exclude pending changes. For more
        information, see `Configuring Availability Options`_ in the
        Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to describe.

        :type deployed: boolean
        :param deployed: Whether to display the deployed configuration (
            `True`) or include any pending changes ( `False`). Defaults to
            `False`.

        """
        params = {'DomainName': domain_name, }
        if deployed is not None:
            params['Deployed'] = str(
                deployed).lower()
        return self._make_request(
            action='DescribeAvailabilityOptions',
            verb='POST',
            path='/', params=params)

    def describe_domains(self, domain_names=None):
        """
        Gets information about the search domains owned by this
        account. Can be limited to specific domains. Shows all domains
        by default. To get the number of searchable documents in a
        domain, use the console or submit a `matchall` request to your
        domain's search endpoint:
        `q=matchall&q.parser=structured&size=0`. For more information,
        see `Getting Information about a Search Domain`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_names: list
        :param domain_names: The names of the domains you want to include in
            the response.

        """
        params = {}
        if domain_names is not None:
            self.build_list_params(params,
                                   domain_names,
                                   'DomainNames.member')
        return self._make_request(
            action='DescribeDomains',
            verb='POST',
            path='/', params=params)

    def describe_expressions(self, domain_name, expression_names=None,
                             deployed=None):
        """
        Gets the expressions configured for the search domain. Can be
        limited to specific expressions by name. By default, shows all
        expressions and includes any pending changes to the
        configuration. Set the `Deployed` option to `True` to show the
        active configuration and exclude pending changes. For more
        information, see `Configuring Expressions`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to describe.

        :type expression_names: list
        :param expression_names: Limits the `DescribeExpressions` response to
            the specified expressions. If not specified, all expressions are
            shown.

        :type deployed: boolean
        :param deployed: Whether to display the deployed configuration (
            `True`) or include any pending changes ( `False`). Defaults to
            `False`.

        """
        params = {'DomainName': domain_name, }
        if expression_names is not None:
            self.build_list_params(params,
                                   expression_names,
                                   'ExpressionNames.member')
        if deployed is not None:
            params['Deployed'] = str(
                deployed).lower()
        return self._make_request(
            action='DescribeExpressions',
            verb='POST',
            path='/', params=params)

    def describe_index_fields(self, domain_name, field_names=None,
                              deployed=None):
        """
        Gets information about the index fields configured for the
        search domain. Can be limited to specific fields by name. By
        default, shows all fields and includes any pending changes to
        the configuration. Set the `Deployed` option to `True` to show
        the active configuration and exclude pending changes. For more
        information, see `Getting Domain Information`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to describe.

        :type field_names: list
        :param field_names: A list of the index fields you want to describe. If
            not specified, information is returned for all configured index
            fields.

        :type deployed: boolean
        :param deployed: Whether to display the deployed configuration (
            `True`) or include any pending changes ( `False`). Defaults to
            `False`.

        """
        params = {'DomainName': domain_name, }
        if field_names is not None:
            self.build_list_params(params,
                                   field_names,
                                   'FieldNames.member')
        if deployed is not None:
            params['Deployed'] = str(
                deployed).lower()
        return self._make_request(
            action='DescribeIndexFields',
            verb='POST',
            path='/', params=params)

    def describe_scaling_parameters(self, domain_name):
        """
        Gets the scaling parameters configured for a domain. A
        domain's scaling parameters specify the desired search
        instance type and replication count. For more information, see
        `Configuring Scaling Options`_ in the Amazon CloudSearch
        Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        """
        params = {'DomainName': domain_name, }
        return self._make_request(
            action='DescribeScalingParameters',
            verb='POST',
            path='/', params=params)

    def describe_service_access_policies(self, domain_name, deployed=None):
        """
        Gets information about the access policies that control access
        to the domain's document and search endpoints. By default,
        shows the configuration with any pending changes. Set the
        `Deployed` option to `True` to show the active configuration
        and exclude pending changes. For more information, see
        `Configuring Access for a Search Domain`_ in the Amazon
        CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to describe.

        :type deployed: boolean
        :param deployed: Whether to display the deployed configuration (
            `True`) or include any pending changes ( `False`). Defaults to
            `False`.

        """
        params = {'DomainName': domain_name, }
        if deployed is not None:
            params['Deployed'] = str(
                deployed).lower()
        return self._make_request(
            action='DescribeServiceAccessPolicies',
            verb='POST',
            path='/', params=params)

    def describe_suggesters(self, domain_name, suggester_names=None,
                            deployed=None):
        """
        Gets the suggesters configured for a domain. A suggester
        enables you to display possible matches before users finish
        typing their queries. Can be limited to specific suggesters by
        name. By default, shows all suggesters and includes any
        pending changes to the configuration. Set the `Deployed`
        option to `True` to show the active configuration and exclude
        pending changes. For more information, see `Getting Search
        Suggestions`_ in the Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: The name of the domain you want to describe.

        :type suggester_names: list
        :param suggester_names: The suggesters you want to describe.

        :type deployed: boolean
        :param deployed: Whether to display the deployed configuration (
            `True`) or include any pending changes ( `False`). Defaults to
            `False`.

        """
        params = {'DomainName': domain_name, }
        if suggester_names is not None:
            self.build_list_params(params,
                                   suggester_names,
                                   'SuggesterNames.member')
        if deployed is not None:
            params['Deployed'] = str(
                deployed).lower()
        return self._make_request(
            action='DescribeSuggesters',
            verb='POST',
            path='/', params=params)

    def index_documents(self, domain_name):
        """
        Tells the search domain to start indexing its documents using
        the latest indexing options. This operation must be invoked to
        activate options whose OptionStatus is
        `RequiresIndexDocuments`.

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        """
        params = {'DomainName': domain_name, }
        return self._make_request(
            action='IndexDocuments',
            verb='POST',
            path='/', params=params)

    def list_domain_names(self):
        """
        Lists all search domains owned by an account.
        """
        params = {}
        return self._make_request(
            action='ListDomainNames',
            verb='POST',
            path='/', params=params)

    def update_availability_options(self, domain_name, multi_az):
        """
        Configures the availability options for a domain. Enabling the
        Multi-AZ option expands an Amazon CloudSearch domain to an
        additional Availability Zone in the same Region to increase
        fault tolerance in the event of a service disruption. Changes
        to the Multi-AZ option can take about half an hour to become
        active. For more information, see `Configuring Availability
        Options`_ in the Amazon CloudSearch Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type multi_az: boolean
        :param multi_az: You expand an existing search domain to a second
            Availability Zone by setting the Multi-AZ option to true.
            Similarly, you can turn off the Multi-AZ option to downgrade the
            domain to a single Availability Zone by setting the Multi-AZ option
            to `False`.

        """
        params = {'DomainName': domain_name, 'MultiAZ': multi_az, }
        return self._make_request(
            action='UpdateAvailabilityOptions',
            verb='POST',
            path='/', params=params)

    def update_scaling_parameters(self, domain_name, scaling_parameters):
        """
        Configures scaling parameters for a domain. A domain's scaling
        parameters specify the desired search instance type and
        replication count. Amazon CloudSearch will still automatically
        scale your domain based on the volume of data and traffic, but
        not below the desired instance type and replication count. If
        the Multi-AZ option is enabled, these values control the
        resources used per Availability Zone. For more information,
        see `Configuring Scaling Options`_ in the Amazon CloudSearch
        Developer Guide .

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type scaling_parameters: dict
        :param scaling_parameters: The desired instance type and desired number
            of replicas of each index partition.

        """
        params = {'DomainName': domain_name, }
        self.build_complex_param(params, 'ScalingParameters',
                                 scaling_parameters)
        return self._make_request(
            action='UpdateScalingParameters',
            verb='POST',
            path='/', params=params)

    def update_service_access_policies(self, domain_name, access_policies):
        """
        Configures the access rules that control access to the
        domain's document and search endpoints. For more information,
        see ` Configuring Access for an Amazon CloudSearch Domain`_.

        :type domain_name: string
        :param domain_name: A string that represents the name of a domain.
            Domain names are unique across the domains owned by an account
            within an AWS region. Domain names start with a letter or number
            and can contain the following characters: a-z (lowercase), 0-9, and
            - (hyphen).

        :type access_policies: string
        :param access_policies: The access rules you want to configure. These
            rules replace any existing rules.

        """
        params = {
            'DomainName': domain_name,
            'AccessPolicies': access_policies,
        }
        return self._make_request(
            action='UpdateServiceAccessPolicies',
            verb='POST',
            path='/', params=params)

    def build_complex_param(self, params, label, value):
        """Serialize a structure.

        For example::

            param_type = 'structure'
            label = 'IndexField'
            value = {'IndexFieldName': 'a', 'IntOptions': {'DefaultValue': 5}}

        would result in the params dict being updated with these params::

            IndexField.IndexFieldName = a
            IndexField.IntOptions.DefaultValue = 5

        :type params: dict
        :param params: The params dict.  The complex list params
            will be added to this dict.

        :type label: str
        :param label: String label for param key

        :type value: any
        :param value: The value to serialize
        """
        for k, v in value.items():
            if isinstance(v, dict):
                for k2, v2 in v.items():
                    self.build_complex_param(params, label + '.' + k, v)
            elif isinstance(v, bool):
                params['%s.%s' % (label, k)] = v and 'true' or 'false'
            else:
                params['%s.%s' % (label, k)] = v

    def _make_request(self, action, verb, path, params):
        params['ContentType'] = 'JSON'
        response = self.make_request(action=action, verb='POST',
                                     path='/', params=params)
        body = response.read().decode('utf-8')
        boto.log.debug(body)
        if response.status == 200:
            return json.loads(body)
        else:
            json_body = json.loads(body)
            fault_name = json_body.get('Error', {}).get('Code', None)
            exception_class = self._faults.get(fault_name, self.ResponseError)
            raise exception_class(response.status, response.reason,
                                  body=json_body)



# Copyright (c) 2006,2007,2008 Mitch Garnaat http://garnaat.org/
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

import datetime
from boto.sdb.db.key import Key
from boto.utils import Password
from boto.sdb.db.query import Query
import re
import boto
import boto.s3.key
from boto.sdb.db.blob import Blob
from boto.compat import six, long_type


class Property(object):

    data_type = str
    type_name = ''
    name = ''
    verbose_name = ''

    def __init__(self, verbose_name=None, name=None, default=None,
                 required=False, validator=None, choices=None, unique=False):
        self.verbose_name = verbose_name
        self.name = name
        self.default = default
        self.required = required
        self.validator = validator
        self.choices = choices
        if self.name:
            self.slot_name = '_' + self.name
        else:
            self.slot_name = '_'
        self.unique = unique

    def __get__(self, obj, objtype):
        if obj:
            obj.load()
            return getattr(obj, self.slot_name)
        else:
            return None

    def __set__(self, obj, value):
        self.validate(value)

        # Fire off any on_set functions
        try:
            if obj._loaded and hasattr(obj, "on_set_%s" % self.name):
                fnc = getattr(obj, "on_set_%s" % self.name)
                value = fnc(value)
        except Exception:
            boto.log.exception("Exception running on_set_%s" % self.name)

        setattr(obj, self.slot_name, value)

    def __property_config__(self, model_class, property_name):
        self.model_class = model_class
        self.name = property_name
        self.slot_name = '_' + self.name

    def default_validator(self, value):
        if isinstance(value, six.string_types) or value == self.default_value():
            return
        if not isinstance(value, self.data_type):
            raise TypeError('Validation Error, %s.%s expecting %s, got %s' % (self.model_class.__name__, self.name, self.data_type, type(value)))

    def default_value(self):
        return self.default

    def validate(self, value):
        if self.required and value is None:
            raise ValueError('%s is a required property' % self.name)
        if self.choices and value and value not in self.choices:
            raise ValueError('%s not a valid choice for %s.%s' % (value, self.model_class.__name__, self.name))
        if self.validator:
            self.validator(value)
        else:
            self.default_validator(value)
        return value

    def empty(self, value):
        return not value

    def get_value_for_datastore(self, model_instance):
        return getattr(model_instance, self.name)

    def make_value_from_datastore(self, value):
        return value

    def get_choices(self):
        if callable(self.choices):
            return self.choices()
        return self.choices


def validate_string(value):
    if value is None:
        return
    elif isinstance(value, six.string_types):
        if len(value) > 1024:
            raise ValueError('Length of value greater than maxlength')
    else:
        raise TypeError('Expecting String, got %s' % type(value))


class StringProperty(Property):

    type_name = 'String'

    def __init__(self, verbose_name=None, name=None, default='',
                 required=False, validator=validate_string,
                 choices=None, unique=False):
        super(StringProperty, self).__init__(verbose_name, name, default, required,
                          validator, choices, unique)


class TextProperty(Property):

    type_name = 'Text'

    def __init__(self, verbose_name=None, name=None, default='',
                 required=False, validator=None, choices=None,
                 unique=False, max_length=None):
        super(TextProperty, self).__init__(verbose_name, name, default, required,
                          validator, choices, unique)
        self.max_length = max_length

    def validate(self, value):
        value = super(TextProperty, self).validate(value)
        if not isinstance(value, six.string_types):
            raise TypeError('Expecting Text, got %s' % type(value))
        if self.max_length and len(value) > self.max_length:
            raise ValueError('Length of value greater than maxlength %s' % self.max_length)


class PasswordProperty(StringProperty):
    """

    Hashed property whose original value can not be
    retrieved, but still can be compared.

    Works by storing a hash of the original value instead
    of the original value.  Once that's done all that
    can be retrieved is the hash.

    The comparison

       obj.password == 'foo'

    generates a hash of 'foo' and compares it to the
    stored hash.

    Underlying data type for hashing, storing, and comparing
    is boto.utils.Password.  The default hash function is
    defined there ( currently sha512 in most cases, md5
    where sha512 is not available )

    It's unlikely you'll ever need to use a different hash
    function, but if you do, you can control the behavior
    in one of two ways:

      1) Specifying hashfunc in PasswordProperty constructor

         import hashlib

         class MyModel(model):
             password = PasswordProperty(hashfunc=hashlib.sha224)

      2) Subclassing Password and PasswordProperty

         class SHA224Password(Password):
             hashfunc=hashlib.sha224

         class SHA224PasswordProperty(PasswordProperty):
             data_type=MyPassword
             type_name="MyPassword"

         class MyModel(Model):
             password = SHA224PasswordProperty()

    """
    data_type = Password
    type_name = 'Password'

    def __init__(self, verbose_name=None, name=None, default='', required=False,
                 validator=None, choices=None, unique=False, hashfunc=None):

        """
           The hashfunc parameter overrides the default hashfunc in boto.utils.Password.

           The remaining parameters are passed through to StringProperty.__init__"""

        super(PasswordProperty, self).__init__(verbose_name, name, default, required,
                                validator, choices, unique)
        self.hashfunc = hashfunc

    def make_value_from_datastore(self, value):
        p = self.data_type(value, hashfunc=self.hashfunc)
        return p

    def get_value_for_datastore(self, model_instance):
        value = super(PasswordProperty, self).get_value_for_datastore(model_instance)
        if value and len(value):
            return str(value)
        else:
            return None

    def __set__(self, obj, value):
        if not isinstance(value, self.data_type):
            p = self.data_type(hashfunc=self.hashfunc)
            p.set(value)
            value = p
        super(PasswordProperty, self).__set__(obj, value)

    def __get__(self, obj, objtype):
        return self.data_type(super(PasswordProperty, self).__get__(obj, objtype), hashfunc=self.hashfunc)

    def validate(self, value):
        value = super(PasswordProperty, self).validate(value)
        if isinstance(value, self.data_type):
            if len(value) > 1024:
                raise ValueError('Length of value greater than maxlength')
        else:
            raise TypeError('Expecting %s, got %s' % (type(self.data_type), type(value)))


class BlobProperty(Property):
    data_type = Blob
    type_name = "blob"

    def __set__(self, obj, value):
        if value != self.default_value():
            if not isinstance(value, Blob):
                oldb = self.__get__(obj, type(obj))
                id = None
                if oldb:
                    id = oldb.id
                b = Blob(value=value, id=id)
                value = b
        super(BlobProperty, self).__set__(obj, value)


class S3KeyProperty(Property):

    data_type = boto.s3.key.Key
    type_name = 'S3Key'
    validate_regex = "^s3:\/\/([^\/]*)\/(.*)$"

    def __init__(self, verbose_name=None, name=None, default=None,
                 required=False, validator=None, choices=None, unique=False):
        super(S3KeyProperty, self).__init__(verbose_name, name, default, required,
                          validator, choices, unique)

    def validate(self, value):
        value = super(S3KeyProperty, self).validate(value)
        if value == self.default_value() or value == str(self.default_value()):
            return self.default_value()
        if isinstance(value, self.data_type):
            return
        match = re.match(self.validate_regex, value)
        if match:
            return
        raise TypeError('Validation Error, expecting %s, got %s' % (self.data_type, type(value)))

    def __get__(self, obj, objtype):
        value = super(S3KeyProperty, self).__get__(obj, objtype)
        if value:
            if isinstance(value, self.data_type):
                return value
            match = re.match(self.validate_regex, value)
            if match:
                s3 = obj._manager.get_s3_connection()
                bucket = s3.get_bucket(match.group(1), validate=False)
                k = bucket.get_key(match.group(2))
                if not k:
                    k = bucket.new_key(match.group(2))
                    k.set_contents_from_string("")
                return k
        else:
            return value

    def get_value_for_datastore(self, model_instance):
        value = super(S3KeyProperty, self).get_value_for_datastore(model_instance)
        if value:
            return "s3://%s/%s" % (value.bucket.name, value.name)
        else:
            return None


class IntegerProperty(Property):

    data_type = int
    type_name = 'Integer'

    def __init__(self, verbose_name=None, name=None, default=0, required=False,
                 validator=None, choices=None, unique=False, max=2147483647, min=-2147483648):
        super(IntegerProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)
        self.max = max
        self.min = min

    def validate(self, value):
        value = int(value)
        value = super(IntegerProperty, self).validate(value)
        if value > self.max:
            raise ValueError('Maximum value is %d' % self.max)
        if value < self.min:
            raise ValueError('Minimum value is %d' % self.min)
        return value

    def empty(self, value):
        return value is None

    def __set__(self, obj, value):
        if value == "" or value is None:
            value = 0
        return super(IntegerProperty, self).__set__(obj, value)


class LongProperty(Property):

    data_type = long_type
    type_name = 'Long'

    def __init__(self, verbose_name=None, name=None, default=0, required=False,
                 validator=None, choices=None, unique=False):
        super(LongProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)

    def validate(self, value):
        value = long_type(value)
        value = super(LongProperty, self).validate(value)
        min = -9223372036854775808
        max = 9223372036854775807
        if value > max:
            raise ValueError('Maximum value is %d' % max)
        if value < min:
            raise ValueError('Minimum value is %d' % min)
        return value

    def empty(self, value):
        return value is None


class BooleanProperty(Property):

    data_type = bool
    type_name = 'Boolean'

    def __init__(self, verbose_name=None, name=None, default=False, required=False,
                 validator=None, choices=None, unique=False):
        super(BooleanProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)

    def empty(self, value):
        return value is None


class FloatProperty(Property):

    data_type = float
    type_name = 'Float'

    def __init__(self, verbose_name=None, name=None, default=0.0, required=False,
                 validator=None, choices=None, unique=False):
        super(FloatProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)

    def validate(self, value):
        value = float(value)
        value = super(FloatProperty, self).validate(value)
        return value

    def empty(self, value):
        return value is None


class DateTimeProperty(Property):
    """This class handles both the datetime.datetime object
    And the datetime.date objects. It can return either one,
    depending on the value stored in the database"""

    data_type = datetime.datetime
    type_name = 'DateTime'

    def __init__(self, verbose_name=None, auto_now=False, auto_now_add=False, name=None,
                 default=None, required=False, validator=None, choices=None, unique=False):
        super(DateTimeProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)
        self.auto_now = auto_now
        self.auto_now_add = auto_now_add

    def default_value(self):
        if self.auto_now or self.auto_now_add:
            return self.now()
        return super(DateTimeProperty, self).default_value()

    def validate(self, value):
        if value is None:
            return
        if isinstance(value, datetime.date):
            return value
        return super(DateTimeProperty, self).validate(value)

    def get_value_for_datastore(self, model_instance):
        if self.auto_now:
            setattr(model_instance, self.name, self.now())
        return super(DateTimeProperty, self).get_value_for_datastore(model_instance)

    def now(self):
        return datetime.datetime.utcnow()


class DateProperty(Property):

    data_type = datetime.date
    type_name = 'Date'

    def __init__(self, verbose_name=None, auto_now=False, auto_now_add=False, name=None,
                 default=None, required=False, validator=None, choices=None, unique=False):
        super(DateProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)
        self.auto_now = auto_now
        self.auto_now_add = auto_now_add

    def default_value(self):
        if self.auto_now or self.auto_now_add:
            return self.now()
        return super(DateProperty, self).default_value()

    def validate(self, value):
        value = super(DateProperty, self).validate(value)
        if value is None:
            return
        if not isinstance(value, self.data_type):
            raise TypeError('Validation Error, expecting %s, got %s' % (self.data_type, type(value)))

    def get_value_for_datastore(self, model_instance):
        if self.auto_now:
            setattr(model_instance, self.name, self.now())
        val = super(DateProperty, self).get_value_for_datastore(model_instance)
        if isinstance(val, datetime.datetime):
            val = val.date()
        return val

    def now(self):
        return datetime.date.today()


class TimeProperty(Property):
    data_type = datetime.time
    type_name = 'Time'

    def __init__(self, verbose_name=None, name=None,
                 default=None, required=False, validator=None, choices=None, unique=False):
        super(TimeProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)

    def validate(self, value):
        value = super(TimeProperty, self).validate(value)
        if value is None:
            return
        if not isinstance(value, self.data_type):
            raise TypeError('Validation Error, expecting %s, got %s' % (self.data_type, type(value)))


class ReferenceProperty(Property):

    data_type = Key
    type_name = 'Reference'

    def __init__(self, reference_class=None, collection_name=None,
                 verbose_name=None, name=None, default=None, required=False, validator=None, choices=None, unique=False):
        super(ReferenceProperty, self).__init__(verbose_name, name, default, required, validator, choices, unique)
        self.reference_class = reference_class
        self.collection_name = collection_name

    def __get__(self, obj, objtype):
        if obj:
            value = getattr(obj, self.slot_name)
            if value == self.default_value():
                return value
            # If the value is still the UUID for the referenced object, we need to create
            # the object now that is the attribute has actually been accessed.  This lazy
            # instantiation saves unnecessary roundtrips to SimpleDB
            if isinstance(value, six.string_types):
                value = self.reference_class(value)
                setattr(obj, self.name, value)
            return value

    def __set__(self, obj, value):
        """Don't allow this object to be associated to itself
        This causes bad things to happen"""
        if value is not None and (obj.id == value or (hasattr(value, "id") and obj.id == value.id)):
            raise ValueError("Can not associate an object with itself!")
        return super(ReferenceProperty, self).__set__(obj, value)

    def __property_config__(self, model_class, property_name):
        super(ReferenceProperty, self).__property_config__(model_class, property_name)
        if self.collection_name is None:
            self.collection_name = '%s_%s_set' % (model_class.__name__.lower(), self.name)
        if hasattr(self.reference_class, self.collection_name):
            raise ValueError('duplicate property: %s' % self.collection_name)
        setattr(self.reference_class, self.collection_name,
                _ReverseReferenceProperty(model_class, property_name, self.collection_name))

    def check_uuid(self, value):
        # This does a bit of hand waving to "type check" the string
        t = value.split('-')
        if len(t) != 5:
            raise ValueError

    def check_instance(self, value):
        try:
            obj_lineage = value.get_lineage()
            cls_lineage = self.reference_class.get_lineage()
            if obj_lineage.startswith(cls_lineage):
                return
            raise TypeError('%s not instance of %s' % (obj_lineage, cls_lineage))
        except:
            raise ValueError('%s is not a Model' % value)

    def validate(self, value):
        if self.validator:
            self.validator(value)
        if self.required and value is None:
            raise ValueError('%s is a required property' % self.name)
        if value == self.default_value():
            return
        if not isinstance(value, six.string_types):
            self.check_instance(value)


class _ReverseReferenceProperty(Property):
    data_type = Query
    type_name = 'query'

    def __init__(self, model, prop, name):
        self.__model = model
        self.__property = prop
        self.collection_name = prop
        self.name = name
        self.item_type = model

    def __get__(self, model_instance, model_class):
        """Fetches collection of model instances of this collection property."""
        if model_instance is not None:
            query = Query(self.__model)
            if isinstance(self.__property, list):
                props = []
                for prop in self.__property:
                    props.append("%s =" % prop)
                return query.filter(props, model_instance)
            else:
                return query.filter(self.__property + ' =', model_instance)
        else:
            return self

    def __set__(self, model_instance, value):
        """Not possible to set a new collection."""
        raise ValueError('Virtual property is read-only')


class CalculatedProperty(Property):

    def __init__(self, verbose_name=None, name=None, default=None,
                 required=False, validator=None, choices=None,
                 calculated_type=int, unique=False, use_method=False):
        super(CalculatedProperty, self).__init__(verbose_name, name, default, required,
                          validator, choices, unique)
        self.calculated_type = calculated_type
        self.use_method = use_method

    def __get__(self, obj, objtype):
        value = self.default_value()
        if obj:
            try:
                value = getattr(obj, self.slot_name)
                if self.use_method:
                    value = value()
            except AttributeError:
                pass
        return value

    def __set__(self, obj, value):
        """Not possible to set a new AutoID."""
        pass

    def _set_direct(self, obj, value):
        if not self.use_method:
            setattr(obj, self.slot_name, value)

    def get_value_for_datastore(self, model_instance):
        if self.calculated_type in [str, int, bool]:
            value = self.__get__(model_instance, model_instance.__class__)
            return value
        else:
            return None


class ListProperty(Property):

    data_type = list
    type_name = 'List'

    def __init__(self, item_type, verbose_name=None, name=None, default=None, **kwds):
        if default is None:
            default = []
        self.item_type = item_type
        super(ListProperty, self).__init__(verbose_name, name, default=default, required=True, **kwds)

    def validate(self, value):
        if self.validator:
            self.validator(value)
        if value is not None:
            if not isinstance(value, list):
                value = [value]

        if self.item_type in six.integer_types:
            item_type = six.integer_types
        elif self.item_type in six.string_types:
            item_type = six.string_types
        else:
            item_type = self.item_type

        for item in value:
            if not isinstance(item, item_type):
                if item_type == six.integer_types:
                    raise ValueError('Items in the %s list must all be integers.' % self.name)
                else:
                    raise ValueError('Items in the %s list must all be %s instances' %
                                     (self.name, self.item_type.__name__))
        return value

    def empty(self, value):
        return value is None

    def default_value(self):
        return list(super(ListProperty, self).default_value())

    def __set__(self, obj, value):
        """Override the set method to allow them to set the property to an instance of the item_type instead of requiring a list to be passed in"""
        if self.item_type in six.integer_types:
            item_type = six.integer_types
        elif self.item_type in six.string_types:
            item_type = six.string_types
        else:
            item_type = self.item_type
        if isinstance(value, item_type):
            value = [value]
        elif value is None:  # Override to allow them to set this to "None" to remove everything
            value = []
        return super(ListProperty, self).__set__(obj, value)


class MapProperty(Property):

    data_type = dict
    type_name = 'Map'

    def __init__(self, item_type=str, verbose_name=None, name=None, default=None, **kwds):
        if default is None:
            default = {}
        self.item_type = item_type
        super(MapProperty, self).__init__(verbose_name, name, default=default, required=True, **kwds)

    def validate(self, value):
        value = super(MapProperty, self).validate(value)
        if value is not None:
            if not isinstance(value, dict):
                raise ValueError('Value must of type dict')

        if self.item_type in six.integer_types:
            item_type = six.integer_types
        elif self.item_type in six.string_types:
            item_type = six.string_types
        else:
            item_type = self.item_type

        for key in value:
            if not isinstance(value[key], item_type):
                if item_type == six.integer_types:
                    raise ValueError('Values in the %s Map must all be integers.' % self.name)
                else:
                    raise ValueError('Values in the %s Map must all be %s instances' %
                                     (self.name, self.item_type.__name__))
        return value

    def empty(self, value):
        return value is None

    def default_value(self):
        return {}



"""
Exceptions that are specific to the cloudtrail module.
"""
from boto.exception import BotoServerError


class InvalidSnsTopicNameException(BotoServerError):
    """
    Raised when an invalid SNS topic name is passed to Cloudtrail.
    """
    pass


class InvalidS3BucketNameException(BotoServerError):
    """
    Raised when an invalid S3 bucket name is passed to Cloudtrail.
    """
    pass


class TrailAlreadyExistsException(BotoServerError):
    """
    Raised when the given trail name already exists.
    """
    pass


class InsufficientSnsTopicPolicyException(BotoServerError):
    """
    Raised when the SNS topic does not allow Cloudtrail to post
    messages.
    """
    pass


class InvalidTrailNameException(BotoServerError):
    """
    Raised when the trail name is invalid.
    """
    pass


class InternalErrorException(BotoServerError):
    """
    Raised when there was an internal Cloudtrail error.
    """
    pass


class TrailNotFoundException(BotoServerError):
    """
    Raised when the given trail name is not found.
    """
    pass


class S3BucketDoesNotExistException(BotoServerError):
    """
    Raised when the given S3 bucket does not exist.
    """
    pass


class TrailNotProvidedException(BotoServerError):
    """
    Raised when no trail name was provided.
    """
    pass


class InvalidS3PrefixException(BotoServerError):
    """
    Raised when an invalid key prefix is given.
    """
    pass


class MaximumNumberOfTrailsExceededException(BotoServerError):
    """
    Raised when no more trails can be created.
    """
    pass


class InsufficientS3BucketPolicyException(BotoServerError):
    """
    Raised when the S3 bucket does not allow Cloudtrail to
    write files into the prefix.
    """
    pass


class InvalidMaxResultsException(BotoServerError):
    pass


class InvalidTimeRangeException(BotoServerError):
    pass


class InvalidLookupAttributesException(BotoServerError):
    pass


class InvalidCloudWatchLogsLogGroupArnException(BotoServerError):
    pass


class InvalidCloudWatchLogsRoleArnException(BotoServerError):
    pass


class CloudWatchLogsDeliveryUnavailableException(BotoServerError):
    pass


class InvalidNextTokenException(BotoServerError):
    pass



# Copyright (c) 2015 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import boto
from boto.compat import json
from boto.connection import AWSQueryConnection
from boto.regioninfo import RegionInfo
from boto.exception import JSONResponseError
from boto.codedeploy import exceptions


class CodeDeployConnection(AWSQueryConnection):
    """
    AWS CodeDeploy **Overview**
    This is the AWS CodeDeploy API Reference. This guide provides
    descriptions of the AWS CodeDeploy APIs. For additional
    information, see the `AWS CodeDeploy User Guide`_.
    **Using the APIs**
    You can use the AWS CodeDeploy APIs to work with the following
    items:


    + Applications , which are unique identifiers that AWS CodeDeploy
      uses to ensure that the correct combinations of revisions,
      deployment configurations, and deployment groups are being
      referenced during deployments. You can work with applications by
      calling CreateApplication, DeleteApplication, GetApplication,
      ListApplications, BatchGetApplications, and UpdateApplication to
      create, delete, and get information about applications, and to
      change information about an application, respectively.
    + Deployment configurations , which are sets of deployment rules
      and deployment success and failure conditions that AWS CodeDeploy
      uses during deployments. You can work with deployment
      configurations by calling CreateDeploymentConfig,
      DeleteDeploymentConfig, GetDeploymentConfig, and
      ListDeploymentConfigs to create, delete, and get information about
      deployment configurations, respectively.
    + Deployment groups , which represent groups of Amazon EC2
      instances to which application revisions can be deployed. You can
      work with deployment groups by calling CreateDeploymentGroup,
      DeleteDeploymentGroup, GetDeploymentGroup, ListDeploymentGroups,
      and UpdateDeploymentGroup to create, delete, and get information
      about single and multiple deployment groups, and to change
      information about a deployment group, respectively.
    + Deployment instances (also known simply as instances ), which
      represent Amazon EC2 instances to which application revisions are
      deployed. Deployment instances are identified by their Amazon EC2
      tags or Auto Scaling group names. Deployment instances belong to
      deployment groups. You can work with deployment instances by
      calling GetDeploymentInstance and ListDeploymentInstances to get
      information about single and multiple deployment instances,
      respectively.
    + Deployments , which represent the process of deploying revisions
      to deployment groups. You can work with deployments by calling
      CreateDeployment, GetDeployment, ListDeployments,
      BatchGetDeployments, and StopDeployment to create and get
      information about deployments, and to stop a deployment,
      respectively.
    + Application revisions (also known simply as revisions ), which
      are archive files that are stored in Amazon S3 buckets or GitHub
      repositories. These revisions contain source content (such as
      source code, web pages, executable files, any deployment scripts,
      and similar) along with an Application Specification file (AppSpec
      file). (The AppSpec file is unique to AWS CodeDeploy; it defines a
      series of deployment actions that you want AWS CodeDeploy to
      execute.) An application revision is uniquely identified by its
      Amazon S3 object key and its ETag, version, or both. Application
      revisions are deployed to deployment groups. You can work with
      application revisions by calling GetApplicationRevision,
      ListApplicationRevisions, and RegisterApplicationRevision to get
      information about application revisions and to inform AWS
      CodeDeploy about an application revision, respectively.
    """
    APIVersion = "2014-10-06"
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "codedeploy.us-east-1.amazonaws.com"
    ServiceName = "codedeploy"
    TargetPrefix = "CodeDeploy_20141006"
    ResponseError = JSONResponseError

    _faults = {
        "InvalidDeploymentIdException": exceptions.InvalidDeploymentIdException,
        "InvalidDeploymentGroupNameException": exceptions.InvalidDeploymentGroupNameException,
        "DeploymentConfigAlreadyExistsException": exceptions.DeploymentConfigAlreadyExistsException,
        "InvalidRoleException": exceptions.InvalidRoleException,
        "RoleRequiredException": exceptions.RoleRequiredException,
        "DeploymentGroupAlreadyExistsException": exceptions.DeploymentGroupAlreadyExistsException,
        "DeploymentConfigLimitExceededException": exceptions.DeploymentConfigLimitExceededException,
        "InvalidNextTokenException": exceptions.InvalidNextTokenException,
        "InvalidDeploymentConfigNameException": exceptions.InvalidDeploymentConfigNameException,
        "InvalidSortByException": exceptions.InvalidSortByException,
        "InstanceDoesNotExistException": exceptions.InstanceDoesNotExistException,
        "InvalidMinimumHealthyHostValueException": exceptions.InvalidMinimumHealthyHostValueException,
        "ApplicationLimitExceededException": exceptions.ApplicationLimitExceededException,
        "ApplicationNameRequiredException": exceptions.ApplicationNameRequiredException,
        "InvalidEC2TagException": exceptions.InvalidEC2TagException,
        "DeploymentDoesNotExistException": exceptions.DeploymentDoesNotExistException,
        "DeploymentLimitExceededException": exceptions.DeploymentLimitExceededException,
        "InvalidInstanceStatusException": exceptions.InvalidInstanceStatusException,
        "RevisionRequiredException": exceptions.RevisionRequiredException,
        "InvalidBucketNameFilterException": exceptions.InvalidBucketNameFilterException,
        "DeploymentGroupLimitExceededException": exceptions.DeploymentGroupLimitExceededException,
        "DeploymentGroupDoesNotExistException": exceptions.DeploymentGroupDoesNotExistException,
        "DeploymentConfigNameRequiredException": exceptions.DeploymentConfigNameRequiredException,
        "DeploymentAlreadyCompletedException": exceptions.DeploymentAlreadyCompletedException,
        "RevisionDoesNotExistException": exceptions.RevisionDoesNotExistException,
        "DeploymentGroupNameRequiredException": exceptions.DeploymentGroupNameRequiredException,
        "DeploymentIdRequiredException": exceptions.DeploymentIdRequiredException,
        "DeploymentConfigDoesNotExistException": exceptions.DeploymentConfigDoesNotExistException,
        "BucketNameFilterRequiredException": exceptions.BucketNameFilterRequiredException,
        "InvalidTimeRangeException": exceptions.InvalidTimeRangeException,
        "ApplicationDoesNotExistException": exceptions.ApplicationDoesNotExistException,
        "InvalidRevisionException": exceptions.InvalidRevisionException,
        "InvalidSortOrderException": exceptions.InvalidSortOrderException,
        "InvalidOperationException": exceptions.InvalidOperationException,
        "InvalidAutoScalingGroupException": exceptions.InvalidAutoScalingGroupException,
        "InvalidApplicationNameException": exceptions.InvalidApplicationNameException,
        "DescriptionTooLongException": exceptions.DescriptionTooLongException,
        "ApplicationAlreadyExistsException": exceptions.ApplicationAlreadyExistsException,
        "InvalidDeployedStateFilterException": exceptions.InvalidDeployedStateFilterException,
        "DeploymentNotStartedException": exceptions.DeploymentNotStartedException,
        "DeploymentConfigInUseException": exceptions.DeploymentConfigInUseException,
        "InstanceIdRequiredException": exceptions.InstanceIdRequiredException,
        "InvalidKeyPrefixFilterException": exceptions.InvalidKeyPrefixFilterException,
        "InvalidDeploymentStatusException": exceptions.InvalidDeploymentStatusException,
    }


    def __init__(self, **kwargs):
        region = kwargs.pop('region', None)
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)

        if 'host' not in kwargs or kwargs['host'] is None:
            kwargs['host'] = region.endpoint

        super(CodeDeployConnection, self).__init__(**kwargs)
        self.region = region

    def _required_auth_capability(self):
        return ['hmac-v4']

    def batch_get_applications(self, application_names=None):
        """
        Gets information about one or more applications.

        :type application_names: list
        :param application_names: A list of application names, with multiple
            application names separated by spaces.

        """
        params = {}
        if application_names is not None:
            params['applicationNames'] = application_names
        return self.make_request(action='BatchGetApplications',
                                 body=json.dumps(params))

    def batch_get_deployments(self, deployment_ids=None):
        """
        Gets information about one or more deployments.

        :type deployment_ids: list
        :param deployment_ids: A list of deployment IDs, with multiple
            deployment IDs separated by spaces.

        """
        params = {}
        if deployment_ids is not None:
            params['deploymentIds'] = deployment_ids
        return self.make_request(action='BatchGetDeployments',
                                 body=json.dumps(params))

    def create_application(self, application_name):
        """
        Creates a new application.

        :type application_name: string
        :param application_name: The name of the application. This name must be
            unique within the AWS user account.

        """
        params = {'applicationName': application_name, }
        return self.make_request(action='CreateApplication',
                                 body=json.dumps(params))

    def create_deployment(self, application_name, deployment_group_name=None,
                          revision=None, deployment_config_name=None,
                          description=None,
                          ignore_application_stop_failures=None):
        """
        Deploys an application revision to the specified deployment
        group.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type deployment_group_name: string
        :param deployment_group_name: The deployment group's name.

        :type revision: dict
        :param revision: The type of revision to deploy, along with information
            about the revision's location.

        :type deployment_config_name: string
        :param deployment_config_name: The name of an existing deployment
            configuration within the AWS user account.
        If not specified, the value configured in the deployment group will be
            used as the default. If the deployment group does not have a
            deployment configuration associated with it, then
            CodeDeployDefault.OneAtATime will be used by default.

        :type description: string
        :param description: A comment about the deployment.

        :type ignore_application_stop_failures: boolean
        :param ignore_application_stop_failures: If set to true, then if the
            deployment causes the ApplicationStop deployment lifecycle event to
            fail to a specific instance, the deployment will not be considered
            to have failed to that instance at that point and will continue on
            to the BeforeInstall deployment lifecycle event.
        If set to false or not specified, then if the deployment causes the
            ApplicationStop deployment lifecycle event to fail to a specific
            instance, the deployment will stop to that instance, and the
            deployment to that instance will be considered to have failed.

        """
        params = {'applicationName': application_name, }
        if deployment_group_name is not None:
            params['deploymentGroupName'] = deployment_group_name
        if revision is not None:
            params['revision'] = revision
        if deployment_config_name is not None:
            params['deploymentConfigName'] = deployment_config_name
        if description is not None:
            params['description'] = description
        if ignore_application_stop_failures is not None:
            params['ignoreApplicationStopFailures'] = ignore_application_stop_failures
        return self.make_request(action='CreateDeployment',
                                 body=json.dumps(params))

    def create_deployment_config(self, deployment_config_name,
                                 minimum_healthy_hosts=None):
        """
        Creates a new deployment configuration.

        :type deployment_config_name: string
        :param deployment_config_name: The name of the deployment configuration
            to create.

        :type minimum_healthy_hosts: dict
        :param minimum_healthy_hosts: The minimum number of healthy instances
            that should be available at any time during the deployment. There
            are two parameters expected in the input: type and value.
        The type parameter takes either of the following values:


        + HOST_COUNT: The value parameter represents the minimum number of
              healthy instances, as an absolute value.
        + FLEET_PERCENT: The value parameter represents the minimum number of
              healthy instances, as a percentage of the total number of instances
              in the deployment. If you specify FLEET_PERCENT, then at the start
              of the deployment AWS CodeDeploy converts the percentage to the
              equivalent number of instances and rounds fractional instances up.


        The value parameter takes an integer.

        For example, to set a minimum of 95% healthy instances, specify a type
            of FLEET_PERCENT and a value of 95.

        """
        params = {'deploymentConfigName': deployment_config_name, }
        if minimum_healthy_hosts is not None:
            params['minimumHealthyHosts'] = minimum_healthy_hosts
        return self.make_request(action='CreateDeploymentConfig',
                                 body=json.dumps(params))

    def create_deployment_group(self, application_name,
                                deployment_group_name,
                                deployment_config_name=None,
                                ec_2_tag_filters=None,
                                auto_scaling_groups=None,
                                service_role_arn=None):
        """
        Creates a new deployment group for application revisions to be
        deployed to.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type deployment_group_name: string
        :param deployment_group_name: The name of an existing deployment group
            for the specified application.

        :type deployment_config_name: string
        :param deployment_config_name: If specified, the deployment
            configuration name must be one of the predefined values, or it can
            be a custom deployment configuration:

        + CodeDeployDefault.AllAtOnce deploys an application revision to up to
              all of the Amazon EC2 instances at once. The overall deployment
              succeeds if the application revision deploys to at least one of the
              instances. The overall deployment fails after the application
              revision fails to deploy to all of the instances. For example, for
              9 instances, deploy to up to all 9 instances at once. The overall
              deployment succeeds if any of the 9 instances is successfully
              deployed to, and it fails if all 9 instances fail to be deployed
              to.
        + CodeDeployDefault.HalfAtATime deploys to up to half of the instances
              at a time (with fractions rounded down). The overall deployment
              succeeds if the application revision deploys to at least half of
              the instances (with fractions rounded up); otherwise, the
              deployment fails. For example, for 9 instances, deploy to up to 4
              instances at a time. The overall deployment succeeds if 5 or more
              instances are successfully deployed to; otherwise, the deployment
              fails. Note that the deployment may successfully deploy to some
              instances, even if the overall deployment fails.
        + CodeDeployDefault.OneAtATime deploys the application revision to only
              one of the instances at a time. The overall deployment succeeds if
              the application revision deploys to all of the instances. The
              overall deployment fails after the application revision first fails
              to deploy to any one instance. For example, for 9 instances, deploy
              to one instance at a time. The overall deployment succeeds if all 9
              instances are successfully deployed to, and it fails if any of one
              of the 9 instances fail to be deployed to. Note that the deployment
              may successfully deploy to some instances, even if the overall
              deployment fails. This is the default deployment configuration if a
              configuration isn't specified for either the deployment or the
              deployment group.


        To create a custom deployment configuration, call the create deployment
            configuration operation.

        :type ec_2_tag_filters: list
        :param ec_2_tag_filters: The Amazon EC2 tags to filter on.

        :type auto_scaling_groups: list
        :param auto_scaling_groups: A list of associated Auto Scaling groups.

        :type service_role_arn: string
        :param service_role_arn: A service role ARN that allows AWS CodeDeploy
            to act on the user's behalf when interacting with AWS services.

        """
        params = {
            'applicationName': application_name,
            'deploymentGroupName': deployment_group_name,
        }
        if deployment_config_name is not None:
            params['deploymentConfigName'] = deployment_config_name
        if ec_2_tag_filters is not None:
            params['ec2TagFilters'] = ec_2_tag_filters
        if auto_scaling_groups is not None:
            params['autoScalingGroups'] = auto_scaling_groups
        if service_role_arn is not None:
            params['serviceRoleArn'] = service_role_arn
        return self.make_request(action='CreateDeploymentGroup',
                                 body=json.dumps(params))

    def delete_application(self, application_name):
        """
        Deletes an application.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        """
        params = {'applicationName': application_name, }
        return self.make_request(action='DeleteApplication',
                                 body=json.dumps(params))

    def delete_deployment_config(self, deployment_config_name):
        """
        Deletes a deployment configuration.

        A deployment configuration cannot be deleted if it is
        currently in use. Also, predefined configurations cannot be
        deleted.

        :type deployment_config_name: string
        :param deployment_config_name: The name of an existing deployment
            configuration within the AWS user account.

        """
        params = {'deploymentConfigName': deployment_config_name, }
        return self.make_request(action='DeleteDeploymentConfig',
                                 body=json.dumps(params))

    def delete_deployment_group(self, application_name,
                                deployment_group_name):
        """
        Deletes a deployment group.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type deployment_group_name: string
        :param deployment_group_name: The name of an existing deployment group
            for the specified application.

        """
        params = {
            'applicationName': application_name,
            'deploymentGroupName': deployment_group_name,
        }
        return self.make_request(action='DeleteDeploymentGroup',
                                 body=json.dumps(params))

    def get_application(self, application_name):
        """
        Gets information about an application.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        """
        params = {'applicationName': application_name, }
        return self.make_request(action='GetApplication',
                                 body=json.dumps(params))

    def get_application_revision(self, application_name, revision):
        """
        Gets information about an application revision.

        :type application_name: string
        :param application_name: The name of the application that corresponds
            to the revision.

        :type revision: dict
        :param revision: Information about the application revision to get,
            including the revision's type and its location.

        """
        params = {
            'applicationName': application_name,
            'revision': revision,
        }
        return self.make_request(action='GetApplicationRevision',
                                 body=json.dumps(params))

    def get_deployment(self, deployment_id):
        """
        Gets information about a deployment.

        :type deployment_id: string
        :param deployment_id: An existing deployment ID within the AWS user
            account.

        """
        params = {'deploymentId': deployment_id, }
        return self.make_request(action='GetDeployment',
                                 body=json.dumps(params))

    def get_deployment_config(self, deployment_config_name):
        """
        Gets information about a deployment configuration.

        :type deployment_config_name: string
        :param deployment_config_name: The name of an existing deployment
            configuration within the AWS user account.

        """
        params = {'deploymentConfigName': deployment_config_name, }
        return self.make_request(action='GetDeploymentConfig',
                                 body=json.dumps(params))

    def get_deployment_group(self, application_name, deployment_group_name):
        """
        Gets information about a deployment group.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type deployment_group_name: string
        :param deployment_group_name: The name of an existing deployment group
            for the specified application.

        """
        params = {
            'applicationName': application_name,
            'deploymentGroupName': deployment_group_name,
        }
        return self.make_request(action='GetDeploymentGroup',
                                 body=json.dumps(params))

    def get_deployment_instance(self, deployment_id, instance_id):
        """
        Gets information about an Amazon EC2 instance as part of a
        deployment.

        :type deployment_id: string
        :param deployment_id: The unique ID of a deployment.

        :type instance_id: string
        :param instance_id: The unique ID of an Amazon EC2 instance in the
            deployment's deployment group.

        """
        params = {
            'deploymentId': deployment_id,
            'instanceId': instance_id,
        }
        return self.make_request(action='GetDeploymentInstance',
                                 body=json.dumps(params))

    def list_application_revisions(self, application_name, sort_by=None,
                                   sort_order=None, s_3_bucket=None,
                                   s_3_key_prefix=None, deployed=None,
                                   next_token=None):
        """
        Lists information about revisions for an application.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type sort_by: string
        :param sort_by: The column name to sort the list results by:

        + registerTime: Sort the list results by when the revisions were
              registered with AWS CodeDeploy.
        + firstUsedTime: Sort the list results by when the revisions were first
              used by in a deployment.
        + lastUsedTime: Sort the list results by when the revisions were last
              used in a deployment.


        If not specified or set to null, the results will be returned in an
            arbitrary order.

        :type sort_order: string
        :param sort_order: The order to sort the list results by:

        + ascending: Sort the list results in ascending order.
        + descending: Sort the list results in descending order.


        If not specified, the results will be sorted in ascending order.

        If set to null, the results will be sorted in an arbitrary order.

        :type s_3_bucket: string
        :param s_3_bucket: A specific Amazon S3 bucket name to limit the search
            for revisions.
        If set to null, then all of the user's buckets will be searched.

        :type s_3_key_prefix: string
        :param s_3_key_prefix: A specific key prefix for the set of Amazon S3
            objects to limit the search for revisions.

        :type deployed: string
        :param deployed:
        Whether to list revisions based on whether the revision is the target
            revision of an deployment group:


        + include: List revisions that are target revisions of a deployment
              group.
        + exclude: Do not list revisions that are target revisions of a
              deployment group.
        + ignore: List all revisions, regardless of whether they are target
              revisions of a deployment group.

        :type next_token: string
        :param next_token: An identifier that was returned from the previous
            list application revisions call, which can be used to return the
            next set of applications in the list.

        """
        params = {'applicationName': application_name, }
        if sort_by is not None:
            params['sortBy'] = sort_by
        if sort_order is not None:
            params['sortOrder'] = sort_order
        if s_3_bucket is not None:
            params['s3Bucket'] = s_3_bucket
        if s_3_key_prefix is not None:
            params['s3KeyPrefix'] = s_3_key_prefix
        if deployed is not None:
            params['deployed'] = deployed
        if next_token is not None:
            params['nextToken'] = next_token
        return self.make_request(action='ListApplicationRevisions',
                                 body=json.dumps(params))

    def list_applications(self, next_token=None):
        """
        Lists the applications registered within the AWS user account.

        :type next_token: string
        :param next_token: An identifier that was returned from the previous
            list applications call, which can be used to return the next set of
            applications in the list.

        """
        params = {}
        if next_token is not None:
            params['nextToken'] = next_token
        return self.make_request(action='ListApplications',
                                 body=json.dumps(params))

    def list_deployment_configs(self, next_token=None):
        """
        Lists the deployment configurations within the AWS user
        account.

        :type next_token: string
        :param next_token: An identifier that was returned from the previous
            list deployment configurations call, which can be used to return
            the next set of deployment configurations in the list.

        """
        params = {}
        if next_token is not None:
            params['nextToken'] = next_token
        return self.make_request(action='ListDeploymentConfigs',
                                 body=json.dumps(params))

    def list_deployment_groups(self, application_name, next_token=None):
        """
        Lists the deployment groups for an application registered
        within the AWS user account.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type next_token: string
        :param next_token: An identifier that was returned from the previous
            list deployment groups call, which can be used to return the next
            set of deployment groups in the list.

        """
        params = {'applicationName': application_name, }
        if next_token is not None:
            params['nextToken'] = next_token
        return self.make_request(action='ListDeploymentGroups',
                                 body=json.dumps(params))

    def list_deployment_instances(self, deployment_id, next_token=None,
                                  instance_status_filter=None):
        """
        Lists the Amazon EC2 instances for a deployment within the AWS
        user account.

        :type deployment_id: string
        :param deployment_id: The unique ID of a deployment.

        :type next_token: string
        :param next_token: An identifier that was returned from the previous
            list deployment instances call, which can be used to return the
            next set of deployment instances in the list.

        :type instance_status_filter: list
        :param instance_status_filter:
        A subset of instances to list, by status:


        + Pending: Include in the resulting list those instances with pending
              deployments.
        + InProgress: Include in the resulting list those instances with in-
              progress deployments.
        + Succeeded: Include in the resulting list those instances with
              succeeded deployments.
        + Failed: Include in the resulting list those instances with failed
              deployments.
        + Skipped: Include in the resulting list those instances with skipped
              deployments.
        + Unknown: Include in the resulting list those instances with
              deployments in an unknown state.

        """
        params = {'deploymentId': deployment_id, }
        if next_token is not None:
            params['nextToken'] = next_token
        if instance_status_filter is not None:
            params['instanceStatusFilter'] = instance_status_filter
        return self.make_request(action='ListDeploymentInstances',
                                 body=json.dumps(params))

    def list_deployments(self, application_name=None,
                         deployment_group_name=None,
                         include_only_statuses=None, create_time_range=None,
                         next_token=None):
        """
        Lists the deployments under a deployment group for an
        application registered within the AWS user account.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type deployment_group_name: string
        :param deployment_group_name: The name of an existing deployment group
            for the specified application.

        :type include_only_statuses: list
        :param include_only_statuses: A subset of deployments to list, by
            status:

        + Created: Include in the resulting list created deployments.
        + Queued: Include in the resulting list queued deployments.
        + In Progress: Include in the resulting list in-progress deployments.
        + Succeeded: Include in the resulting list succeeded deployments.
        + Failed: Include in the resulting list failed deployments.
        + Aborted: Include in the resulting list aborted deployments.

        :type create_time_range: dict
        :param create_time_range: A deployment creation start- and end-time
            range for returning a subset of the list of deployments.

        :type next_token: string
        :param next_token: An identifier that was returned from the previous
            list deployments call, which can be used to return the next set of
            deployments in the list.

        """
        params = {}
        if application_name is not None:
            params['applicationName'] = application_name
        if deployment_group_name is not None:
            params['deploymentGroupName'] = deployment_group_name
        if include_only_statuses is not None:
            params['includeOnlyStatuses'] = include_only_statuses
        if create_time_range is not None:
            params['createTimeRange'] = create_time_range
        if next_token is not None:
            params['nextToken'] = next_token
        return self.make_request(action='ListDeployments',
                                 body=json.dumps(params))

    def register_application_revision(self, application_name, revision,
                                      description=None):
        """
        Registers with AWS CodeDeploy a revision for the specified
        application.

        :type application_name: string
        :param application_name: The name of an existing AWS CodeDeploy
            application within the AWS user account.

        :type description: string
        :param description: A comment about the revision.

        :type revision: dict
        :param revision: Information about the application revision to
            register, including the revision's type and its location.

        """
        params = {
            'applicationName': application_name,
            'revision': revision,
        }
        if description is not None:
            params['description'] = description
        return self.make_request(action='RegisterApplicationRevision',
                                 body=json.dumps(params))

    def stop_deployment(self, deployment_id):
        """
        Attempts to stop an ongoing deployment.

        :type deployment_id: string
        :param deployment_id: The unique ID of a deployment.

        """
        params = {'deploymentId': deployment_id, }
        return self.make_request(action='StopDeployment',
                                 body=json.dumps(params))

    def update_application(self, application_name=None,
                           new_application_name=None):
        """
        Changes an existing application's name.

        :type application_name: string
        :param application_name: The current name of the application that you
            want to change.

        :type new_application_name: string
        :param new_application_name: The new name that you want to change the
            application to.

        """
        params = {}
        if application_name is not None:
            params['applicationName'] = application_name
        if new_application_name is not None:
            params['newApplicationName'] = new_application_name
        return self.make_request(action='UpdateApplication',
                                 body=json.dumps(params))

    def update_deployment_group(self, application_name,
                                current_deployment_group_name,
                                new_deployment_group_name=None,
                                deployment_config_name=None,
                                ec_2_tag_filters=None,
                                auto_scaling_groups=None,
                                service_role_arn=None):
        """
        Changes information about an existing deployment group.

        :type application_name: string
        :param application_name: The application name corresponding to the
            deployment group to update.

        :type current_deployment_group_name: string
        :param current_deployment_group_name: The current name of the existing
            deployment group.

        :type new_deployment_group_name: string
        :param new_deployment_group_name: The new name of the deployment group,
            if you want to change it.

        :type deployment_config_name: string
        :param deployment_config_name: The replacement deployment configuration
            name to use, if you want to change it.

        :type ec_2_tag_filters: list
        :param ec_2_tag_filters: The replacement set of Amazon EC2 tags to
            filter on, if you want to change them.

        :type auto_scaling_groups: list
        :param auto_scaling_groups: The replacement list of Auto Scaling groups
            to be included in the deployment group, if you want to change them.

        :type service_role_arn: string
        :param service_role_arn: A replacement service role's ARN, if you want
            to change it.

        """
        params = {
            'applicationName': application_name,
            'currentDeploymentGroupName': current_deployment_group_name,
        }
        if new_deployment_group_name is not None:
            params['newDeploymentGroupName'] = new_deployment_group_name
        if deployment_config_name is not None:
            params['deploymentConfigName'] = deployment_config_name
        if ec_2_tag_filters is not None:
            params['ec2TagFilters'] = ec_2_tag_filters
        if auto_scaling_groups is not None:
            params['autoScalingGroups'] = auto_scaling_groups
        if service_role_arn is not None:
            params['serviceRoleArn'] = service_role_arn
        return self.make_request(action='UpdateDeploymentGroup',
                                 body=json.dumps(params))

    def make_request(self, action, body):
        headers = {
            'X-Amz-Target': '%s.%s' % (self.TargetPrefix, action),
            'Host': self.region.endpoint,
            'Content-Type': 'application/x-amz-json-1.1',
            'Content-Length': str(len(body)),
        }
        http_request = self.build_base_http_request(
            method='POST', path='/', auth_path='/', params={},
            headers=headers, data=body)
        response = self._mexe(http_request, sender=None,
                              override_num_retries=10)
        response_body = response.read().decode('utf-8')
        boto.log.debug(response_body)
        if response.status == 200:
            if response_body:
                return json.loads(response_body)
        else:
            json_body = json.loads(response_body)
            fault_name = json_body.get('__type', None)
            exception_class = self._faults.get(fault_name, self.ResponseError)
            raise exception_class(response.status, response.reason,
                                  body=json_body)




# Copyright (c) 2006-2009 Mitch Garnaat http://garnaat.org/
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

"""
Represents an EC2 Spot Instance Request
"""

from boto.ec2.ec2object import EC2Object


class SpotPriceHistory(EC2Object):

    def __init__(self, connection=None):
        super(SpotPriceHistory, self).__init__(connection)
        self.price = 0.0
        self.instance_type = None
        self.product_description = None
        self.timestamp = None
        self.availability_zone = None

    def __repr__(self):
        return 'SpotPriceHistory(%s):%2f' % (self.instance_type, self.price)

    def endElement(self, name, value, connection):
        if name == 'instanceType':
            self.instance_type = value
        elif name == 'spotPrice':
            self.price = float(value)
        elif name == 'productDescription':
            self.product_description = value
        elif name == 'timestamp':
            self.timestamp = value
        elif name == 'availabilityZone':
            self.availability_zone = value
        else:
            setattr(self, name, value)
